﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 画像群を管理するクラスです。
    /// </summary>
    class ImageManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static ImageManager manager = new ImageManager();

        /// <summary>
        /// ボード画像。
        /// </summary>
        public int[] img_board = new int[40];

        /// <summary>
        /// キャラクタ画像 9枚分。X2(変身用)。
        /// </summary>
        public int[,] img_ch = new int[2, 12];

        /// <summary>
        /// 敵画像 9枚分。X3体分。
        /// </summary>
        public int[,] img_enemy = new int[3, 9];

        /// <summary>
        /// 弾の画像。
        /// </summary>
        public int[,] img_bullet = new int[10, 10];

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private ImageManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static ImageManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// 画像を読み込みます。
        /// </summary>
        public void load()
        {
            string baseDir = Setting.BASE_DIR;
            this.img_board[10] = DX.LoadGraph(baseDir + "img/board/10.png");
            this.img_board[11] = DX.LoadGraph(baseDir + "img/board/11.png");
            this.img_board[12] = DX.LoadGraph(baseDir + "img/board/12.png");
            this.img_board[20] = DX.LoadGraph(baseDir + "img/board/20.png");
            DX.LoadDivGraph(baseDir + "img/char/0.png", 12, 4, 3, 73, 73, out this.img_ch[0, 0]);
            DX.LoadDivGraph(baseDir + "img/enemy/0.png", 9, 3, 3, 32, 32, out this.img_enemy[0, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b0.png", 5, 5, 1, 76, 76, out this.img_bullet[0, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b1.png", 6, 6, 1, 22, 22, out this.img_bullet[1, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b2.png", 10, 10, 1, 5, 120, out this.img_bullet[2, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b3.png", 5, 5, 1, 19, 34, out this.img_bullet[3, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b4.png", 10, 10, 1, 38, 38, out this.img_bullet[4, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b5.png", 3, 3, 1, 14, 16, out this.img_bullet[5, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b6.png", 3, 3, 1, 14, 18, out this.img_bullet[6, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b7.png", 9, 9, 1, 16, 16, out this.img_bullet[7, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b8.png", 10, 10, 1, 12, 18, out this.img_bullet[8, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b9.png", 3, 3, 1, 13, 19, out this.img_bullet[9, 0]);
        }
    }
}
