﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵の弾幕を表すクラスです。
    /// </summary>
    class Shot
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// どの敵から発射されたか。
        /// </summary>
        public Enemy parent_enemy;

        /// <summary>
        /// ベース角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// ベーススピード。
        /// </summary>
        public double[] base_spd = new double[1];

        /// <summary>
        /// 弾群。
        /// </summary>
        public BulletGroup bullets = new BulletGroup();

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.parent_enemy = null;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0;
            for (int i = 0; i < this.base_spd.Length; i++)
                this.base_spd[i] = 0;
            this.bullets.initialize();
        }
        
        /// <summary>
        /// 弾を発射した敵と自キャラクタとの角度を返します。
        /// </summary>
        /// <returns>角度</returns>
        private double shotatan2()
        {
            return Math.Atan2(GV.ch.y - this.parent_enemy.y, GV.ch.x - this.parent_enemy.x);
        }

        /// <summary>
        /// 1発だけ自キャラクタに向かって直線移動。
        /// </summary>
        private void shot_bullet_H000()
        {
            if (this.cnt == 0)
            {
                if (this.flag != 2)
                {
                    Bullet bullet = this.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = this.parent_enemy.blknd2;
                        bullet.angle = this.shotatan2();
                        bullet.flag = 1;
                        bullet.x = this.parent_enemy.x;
                        bullet.y = this.parent_enemy.y;
                        bullet.col = this.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕を処理します。
        /// </summary>
        public void do_shot_bullet()
        {
            switch (this.knd)
            {
                case 0:
                    this.shot_bullet_H000();
                    break;
                default:
                    Debug.WriteLine("弾幕の種類が異常値(" + this.knd + ")です");
                    break;
            }

        }
        
        /// <summary>
        /// 弾幕を計算します。
        /// </summary>
        public void calc()
        {
            if (this.parent_enemy.flag != 1) // 敵が倒されたら
                this.flag = 2; // それ以上弾を登録しないフラグに変える
            this.bullets.calc(); // 弾群について計算する

            if (this.bullets.exist_display_bullet()) // 表示中の弾がひとつでもあれば
                return; // 処理を終える

            // 表示中の弾がひとつも無い場合
            if (this.parent_enemy.flag != 1) // 敵が倒されたら
            {
                this.flag = 0; // 終了
                this.parent_enemy.flag = 0; // 処理不明
            }
        }
    }
}
