﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵群を管理するクラスです。
    /// </summary>
    class EnemyGroup
    {
        /// <summary>
        /// 敵群。
        /// </summary>
        private Enemy[] enemys = new Enemy[Setting.ENEMY_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public EnemyGroup()
        {
            for (int i = 0; i < this.enemys.Length; i++)
            {
                this.enemys[i] = new Enemy();
            }
        }

        /// <summary>
        /// 敵にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>敵</returns>
        public Enemy this[int index]
        {
            get
            {
                return this.enemys[index];
            }
        }

        /// <summary>
        /// 敵の数を返します。
        /// </summary>
        /// <returns>敵の数</returns>
        public int length()
        {
            return this.enemys.Length;
        }

        /// <summary>
        /// 空いている敵を検索します。
        /// </summary>
        /// <returns>空いている敵インスタンス。すべて埋まっていた場合は null</returns>
        private Enemy enemy_search()
        {
            for (int i = 0; i < this.enemys.Length; i++)
            {
                Enemy enemy = this.enemys[i];
                if (enemy.flag == 0)
                    return enemy; // 使用可能な敵インスタンスを返す
            }

            return null; // 全部埋まっていたら nullを返す
        }

        /// <summary>
        /// 敵データを登録します。
        /// </summary>
        private void enemy_enter()
        {
            for (int t = 0; t < GV.enemy_orders.length(); t++)
            {
                EnemyOrder enemy_order = GV.enemy_orders[t];
                if (enemy_order.cnt == GV.stage_count) // 現在の瞬間がオーダーの瞬間なら
                {
                    Enemy enemy = this.enemy_search();
                    if (enemy != null)
                    {
                        enemy.flag = 1; // フラグ
                        enemy.cnt = 0; // カウンタ
                        enemy.pattern = enemy_order.pattern; // 移動パターン
                        enemy.muki = 1; // 向き
                        enemy.knd = enemy_order.knd; // 敵の種類
                        enemy.x = enemy_order.x; // X座標
                        enemy.y = enemy_order.y; // Y座標
                        enemy.sp = enemy_order.sp; // スピード
                        enemy.bltime = enemy_order.bltime; // 弾の発射時間
                        enemy.blknd = enemy_order.blknd; // 弾幕の種類
                        enemy.blknd2 = enemy_order.blknd2; // 弾の種類
                        enemy.col = enemy_order.col; // 色
                        enemy.wait = enemy_order.wait; // 停滞時間
                        enemy.hp = enemy_order.hp; // 体力
                        enemy.hp_max = enemy.hp; // 体力最大値
                        enemy.vx = 0.0; // 水平成分の速度
                        enemy.vy = 0.0; // 垂直成分の速度
                        enemy.ang = 0.0;
                        for (int j = 0; j < enemy.item_n.Length; j++)
                            enemy.item_n[j] = enemy_order.item_n[j]; // 落とすアイテム
                    }
                }

            }
        }

        /// <summary>
        /// 敵の行動制御をします。
        /// </summary>
        private void enemy_act()
        {
            for (int i = 0; i < this.enemys.Length; i++)
            {
                Enemy enemy = this.enemys[i];
                if(enemy.flag == 1) // その敵のフラグがオンになっていたら
                {
                    enemy.do_pattern();
                    enemy.x += Math.Cos(enemy.ang) * enemy.sp;
                    enemy.y += Math.Sin(enemy.ang) * enemy.sp;
                    enemy.x += enemy.vx;
                    enemy.y += enemy.vy;
                    enemy.cnt++;
                    enemy.img = enemy.muki * 3 + (enemy.cnt % 18) / 6;

                    // 敵が画面から外れたら消す
                    if (enemy.x < -50 || Setting.FIELD_MAX_X + 50 < enemy.x || enemy.y < -50 || Setting.FIELD_MAX_Y + 50 < enemy.y)
                        enemy.flag = 0;

                    if (enemy.bltime == enemy.cnt)
                        enemy.enter_shot();
                }
            }
        }

        /// <summary>
        /// 敵処理メイン
        /// </summary>
        public void enemy_main()
        {
            enemy_enter();
            enemy_act();
        }
    }
}
