﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// キャラクターのショットを表すクラスです。
    /// </summary>
    class CShot
    {
        /// <summary>
        /// 敵の当たり判定範囲。
        /// TODO 暫定でここに置きます。
        /// </summary>
        private int[] enemy_range = new int[] { 16, 30, 16, 50 };

        /// <summary>
        /// 自キャラクタのショットの当たり判定範囲。
        /// </summary>
        private int[] cshot_range = new int[] { 6, 0 };

        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// パワー。
        /// </summary>
        public int power;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;
        
        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 角度。
        /// </summary>
        public double angle;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.power = 0;
            this.cnt = 0;
            this.knd = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.angle = 0.0;
            this.spd = 0.0;
        }
        
        /// <summary>
        /// 当たったかどうか判定する。
        /// </summary>
        /// <param name="enemy">敵</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_cshot(Enemy enemy)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                double x = this.x - enemy.x; // 敵と自キャラクタのショットとの差
                double y = this.y - enemy.y;

                // オーバーフロー対策
                if (this.knd >= cshot_range.Length || enemy.knd >= enemy_range.Length)
                {
                    Debug.WriteLine("out_judge_cshot内オーバーフロー");
                    return false;
                }

                // 敵の当たり判定と自キャラクタのショットの当たり判定の合計範囲
                double r = cshot_range[this.knd] + enemy_range[enemy.knd];

                // 中間を計算する必要があれば
                if (this.spd > r)
                {
                    // 1フレーム前にいた位置を格納する
                    double pre_x = this.x + Math.Cos(this.angle + Math.PI) * this.spd;
                    double pre_y = this.y + Math.Sin(this.angle + Math.PI) * this.spd;
                    double px;
                    double py;

                    for (int j = 0; j < (int)(this.spd / r); j++) // 進んだ分÷当たり判定分ループ
                    {
                        // 敵と自キャラクタのショットとの差
                        px = pre_x - enemy.x;
                        py = pre_y - enemy.y;

                        if (px * px + py * py < r * r) // 当たり範囲内なら
                            return true; // 当たり

                        // 調べる範囲を進める
                        pre_x += Math.Cos(this.angle) * r;
                        pre_y += Math.Sin(this.angle) * r;
                    }
                }

                if (x * x + y * y < r * r) // 当たり判定内なら
                    return true; // 当たり
            }

            return false;
        }

        /// <summary>
        /// 敵が死ぬかどうかの判定をします。
        /// </summary>
        /// <param name="enemy">敵</param>
        private void enemy_death_judge(Enemy enemy)
        {
            MusicManager musicManager = MusicManager.GetInstance();
            musicManager.se_flag[8] = 1; // 敵に当たった音

            if(enemy.hp < 0) // 敵の HPが 0未満になったら
            {
                musicManager.se_flag[1] = 1; // 敵のピチュリ音
                enemy.death(); // 敵を消滅させる処理(敵弾の消滅処理も行う)
            }
        }

        /// <summary>
        /// 当たり判定メイン(ショット単位)。
        /// </summary>
        public void out_main()
        {
            for (int s = 0; s < GV.enemys.length(); s++) // 敵の総数
            {
                Enemy enemy = GV.enemys[s];
                if (enemy.flag > 0)
                {
                    if (this.out_judge_cshot(enemy)) // 自キャラクタのショットと敵が当たっていたら
                    {
                        this.flag = 0; // 自キャラクタのショットを消す
                        enemy.decrease_hp(this); // 弾の持つパワー分 HPを減らす
                        this.enemy_death_judge(enemy); // 敵が死ぬかどうかを決める
                    }
                }
            }
        }
    }
}
