﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵を表すクラスです。
    /// </summary>
    class Enemy
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 移動パターン。
        /// </summary>
        public int pattern;

        /// <summary>
        /// 向き。
        /// </summary>
        public int muki;

        /// <summary>
        /// 敵の種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// HP。
        /// </summary>
        public int hp;

        /// <summary>
        /// HP最大値。
        /// </summary>
        public int hp_max;

        /// <summary>
        /// 落とすアイテム。
        /// </summary>
        public int[] item_n = new int[6];

        /// <summary>
        /// 画像。
        /// </summary>
        public int img;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 速度 x成分。
        /// </summary>
        public double vx;

        /// <summary>
        /// 速度 y成分。
        /// </summary>
        public double vy;

        /// <summary>
        /// スピード。
        /// </summary>
        public double sp;

        /// <summary>
        /// 角度。
        /// </summary>
        public double ang;

        /// <summary>
        /// 弾幕開始時間。
        /// </summary>
        public int bltime;

        /// <summary>
        /// 弾幕の種類。
        /// </summary>
        public int blknd;

        /// <summary>
        /// 弾の種類。
        /// </summary>
        public int blknd2;

        /// <summary>
        /// 色。
        /// </summary>
        public int col;

        /// <summary>
        /// 状態。
        /// </summary>
        public int state;

        /// <summary>
        /// 待機時間。
        /// </summary>
        public int wtime;

        /// <summary>
        /// 停滞時間。
        /// </summary>
        public int wait;

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.cnt = 0;
            this.pattern = 0;
            this.muki = 0;
            this.knd = 0;
            this.hp = 0;
            this.hp_max = 0;
            for (int i = 0; i < item_n.Length; i++)
                this.item_n[i] = 0;
            this.img = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.vx = 0.0;
            this.vy = 0.0;
            this.sp = 0.0;
            this.ang = 0.0;
            this.bltime = 0;
            this.blknd2 = 0;
            this.col = 0;
            this.state = 0;
            this.wtime = 0;
            this.wait = 0;
        }

        /// <summary>
        /// 移動パターン 0。
        /// 下がってきて停滞して上がっていく。
        /// </summary>
        private void pattern0()
        {
            if (this.cnt == 0)
                this.vy = 3.0; // 下がってくる
            if (this.cnt == 40)
                this.vy = 0.0; // 止まる
            if (this.cnt == 40 + this.wait) // 登録された時間だけ停滞して
                this.vy = -3.0; // 上がっていく
        }
        
        /// <summary>
        /// 移動パターン 1。
        /// 下がってきて停滞して左下に行く。
        /// </summary>
        private void pattern1()
        {
            if (this.cnt == 0)
                this.vy = 3.0; // 下がってくる
            if (this.cnt == 40)
                this.vy = 0.0; // 止まる
            if (this.cnt == 40 + this.wait) // 登録された時間だけ停滞して
            {
                this.vx = -1.0; // 左へ下がっていく
                this.vy = 2.0;
                this.muki = 0; // 左向きセット
            }
        }
        
        /// <summary>
        /// 移動パターン 2。
        /// 下がってきて停滞して右下に行く。
        /// </summary>
        private void pattern2()
        {
            if (this.cnt == 0)
                this.vy = 3.0; // 下がってくる
            if (this.cnt == 40)
                this.vy = 0.0; // 止まる
            if (this.cnt == 40 + this.wait) // 登録された時間だけ停滞して
            {
                this.vx = 1.0; // 右へ下がっていく
                this.vy = 2.0;
                this.muki = 0; // 右向きセット
            }
        }

        /// <summary>
        /// 移動パターン 3。
        /// すばやく降りてきて左へ。
        /// </summary>
        private void pattern3()
        {
            if (this.cnt == 0)
                this.vy = 5.0;
            if (this.cnt == 30) // 途中で左向きに
                this.muki = 0;
            if (this.cnt < 100)
            {
                this.vx -= 5.0 / 100.0; // 左向き加速
                this.vy -= 5.0 / 100.0; // 減速
            }
        }

        /// <summary>
        /// 移動パターン 4。
        /// すばやく降りてきて右へ。
        /// </summary>
        private void pattern4()
        {
            if (this.cnt == 0)
                this.vy = 5.0;
            if (this.cnt == 30) // 途中で右向きに
                this.muki = 2;
            if (this.cnt < 100)
            {
                this.vx += 5.0 / 100.0; // 右向き加速
                this.vy -= 5.0 / 100.0; // 減速
            }
        }

        /// <summary>
        /// 移動パターン 5。
        /// 斜め左下へ。
        /// </summary>
        private void pattern5()
        {
            if (this.cnt == 0)
            {
                this.vx -= 1.0;
                this.vy = 2.0;
                this.muki = 0;
            }
        }

        /// <summary>
        /// 移動パターン 6。
        /// 斜め右下へ。
        /// </summary>
        private void pattern6()
        {
            if (this.cnt == 0)
            {
                this.vx += 1.0;
                this.vy = 2.0;
                this.muki = 2;
            }
        }

        /// <summary>
        /// 移動パターン 7。
        /// 停滞してそのまま左上に。
        /// </summary>
        private void pattern7()
        {
            if (this.cnt == this.wait) // 登録された時間だけ停滞して
            {
                this.vx = -0.7; // 左へ上がっていく
                this.vy = -0.3;
                this.muki = 0; // 左向き
            }
        }

        /// <summary>
        /// 移動パターン 8。
        /// 停滞してそのまま右上に。
        /// </summary>
        private void pattern8()
        {
            if (this.cnt == this.wait) // 登録された時間だけ停滞して
            {
                this.vx = 0.7; // 右へ上がっていく
                this.vy = -0.3;
                this.muki = 2; // 右向き
            }
        }

        /// <summary>
        /// 移動パターン 9。
        /// 停滞してそのまま上に。
        /// </summary>
        private void pattern9()
        {
            if (this.cnt == this.wait) // 登録された時間だけ停滞して
            {
                this.vy = -1.0;
            }
        }

        /// <summary>
        /// 下がってきてウロウロして上がっていく。
        /// </summary>
        private void pattern10()
        {
            if (this.cnt == 0)
                this.vy = 4.0; // 下がってくる
            if (this.cnt == 40)
                this.vy = 0.0; // 止まる
            if (this.cnt >= 40)
            {
                if (this.cnt % 60 == 0)
                {
                    int r = (Math.Cos(this.ang) < 0.0) ? 0 : 1;
                    this.sp = 6.0 + MyMath.rang(2.0);
                    this.ang = MyMath.rang(Math.PI / 4.0) + Math.PI * r;
                    this.muki = 2 - 2 * r;
                }
                this.sp *= 0.95;
            }
            if (this.cnt >= 40 + this.wait)
                this.vy -= 0.05;
        }

        /// <summary>
        /// パターンを実行します。
        /// </summary>
        public void do_pattern()
        {
            switch (this.pattern)
            {
                case 0:
                    this.pattern0();
                    break;
                case 1:
                    this.pattern1();
                    break;
                case 2:
                    this.pattern2();
                    break;
                case 3:
                    this.pattern3();
                    break;
                case 4:
                    this.pattern4();
                    break;
                case 5:
                    this.pattern5();
                    break;
                case 6:
                    this.pattern6();
                    break;
                case 7:
                    this.pattern7();
                    break;
                case 8:
                    this.pattern8();
                    break;
                case 9:
                    this.pattern9();
                    break;
                case 10:
                    this.pattern10();
                    break;
                default:
                    Debug.WriteLine("Enemyの patternの値が " + this.pattern + "です。");
                    break;
            }
        }

        /// <summary>
        /// 弾幕を登録します。
        /// </summary>
        public void enter_shot()
        {
            Shot shot = GV.shots.search_unused_shot(); // フラグの立ってない弾幕を探す
            if (shot != null) // 未使用の弾幕があれば
            {
                shot.initialize(); // 初期化して登録する
                shot.flag = 1; // フラグを立てる
                shot.knd = this.blknd; // 弾の種類
                shot.parent_enemy = this; // どの敵から発射されたものか。
                shot.cnt = 0;
            }
        }

        /// <summary>
        /// 敵の体力を減少させます。
        /// </summary>
        /// <param name="shot">ショット</param>
        public void decrease_hp(CShot cshot)
        {
            this.hp -= cshot.power;
        }

        /// <summary>
        /// 敵の消滅処理を行います。同時にショットの消滅処理を行います。
        /// </summary>
        public void death()
        {
            this.flag = 0;
            GV.shots.death(this);
        }
    }
}
