﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// グラフィックスを管理するクラスです。
    /// </summary>
    class GraphManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static GraphManager manager = new GraphManager();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GraphManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static GraphManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// 敵キャラクタ群を描画します。
        /// </summary>
        private void graph_enemy()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.enemys.length(); i++)
            {
                Enemy enemy = GV.enemys[i];
                if (enemy.flag == 1)
                    DX.DrawRotaGraphF((float)enemy.x + Setting.FIELD_X, (float)enemy.y + Setting.FIELD_Y, 1.0, 0.0, imageManager.img_enemy[0, enemy.img], 1);
            }
        }

        private void graph_cshot()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.cshots.length(); i++)
            {
                CShot cshot = GV.cshots[i];
                if (cshot.flag > 0)
                    DX.DrawRotaGraphF((float)cshot.x + Setting.FIELD_X, (float)cshot.y + Setting.FIELD_Y, 1.0, 0.0, imageManager.img_cshot[cshot.knd], 1);
            }
        }

        /// <summary>
        /// 自キャラクタを描画します。
        /// </summary>
        private void graph_ch()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawRotaGraphF((float)GV.ch.x + Setting.FIELD_X, (float)GV.ch.y + Setting.FIELD_Y, 1.0, 0.0, imageManager.img_ch[0, GV.ch.img], 1);
        }

        /// <summary>
        /// 弾幕を描画します。
        /// </summary>
        private void graph_bullet()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawMode(DX.DX_DRAWMODE_BILINEAR); // 線形補間描画
            for (int i = 0; i < GV.shots.length(); i++) // 弾幕数分ループ
            {
                Shot shot = GV.shots[i];
                if (shot.flag > 0) // 弾幕データがオンなら
                {
                    BulletGroup bullets = shot.bullets;
                    for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                    {
                        Bullet bullet = bullets[j];
                        if (bullet.flag != 0) // 弾データがオンなら
                        {
                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                            DX.DrawRotaGraphF((float)bullet.x + Setting.FIELD_X, (float)bullet.y + Setting.FIELD_Y,
                                1.0, bullet.angle + Math.PI / 2.0,
                                imageManager.img_bullet[bullet.knd, bullet.col], 1);

                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                        }
                    }
                }
            }
            DX.SetDrawMode(DX.DX_DRAWMODE_NEAREST); // 描画形式を戻す
        }

        /// <summary>
        /// ボードを描画します。
        /// </summary>
        private void graph_board()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawGraph(0, 0, imageManager.img_board[10], 0);
            DX.DrawGraph(0, 16, imageManager.img_board[11], 0);
            DX.DrawGraph(0, 464, imageManager.img_board[12], 0);
            DX.DrawGraph(416, 0, imageManager.img_board[20], 0);
        }

        /// <summary>
        /// 描画します。
        /// </summary>
        public void graph_main()
        {
            this.graph_enemy();
            this.graph_cshot();
            this.graph_ch();
            this.graph_bullet();
            this.graph_board();
        }
    }
}
