﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// キー入力を管理するためのクラスです。
    /// </summary>
    class KeyManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static KeyManager manager = new KeyManager();

        /// <summary>
        /// キー状態群。
        /// </summary>
        private int[] stateKey = new int[256];

        /// <summary>
        /// パッドの入力状態。
        /// </summary>
        private Pad pad = new Pad();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private KeyManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static KeyManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// キー状態を取得します。
        /// </summary>
        /// <returns>常に 0</returns>
        public int GetHitKeyStateAll_2()
        {
            byte[] GetHitKeyStateAll_Key = new byte[256];
            DX.GetHitKeyStateAll(out GetHitKeyStateAll_Key[0]);
            for (int i = 0; i < GetHitKeyStateAll_Key.Length; i++)
            {
                if (GetHitKeyStateAll_Key[i] == 1)
                    this.stateKey[i] ++;
                else
                    this.stateKey[i] = 0;
            }

            return 0;
        }

        /// <summary>
        /// 受け取ったキー番号の現在の入力状態を返します。
        /// </summary>
        /// <param name="handle">キー番号</param>
        /// <returns>現在の入力状態</returns>
        public int CheckStateKey(int handle)
        {
            return this.stateKey[handle];
        }

        /// <summary>
        /// 引数 1と引数 2のうち、大きいほうを引数 1に代入します。
        /// </summary>
        /// <param name="p">引数 1</param>
        /// <param name="k">引数 2</param>
        private void input_pad_or_key(ref int p, int k)
        {
            p = (p > k) ? p : k;
        }
        
        /// <summary>
        /// パッドとキーボードの両方の入力をチェックします。
        /// </summary>
        public void GetHitPadStateAll()
        {
            int padInput;
            int mul = 1;

            padInput = DX.GetJoypadInputState(DX.DX_INPUT_PAD1); // パッドの入力状態を取得

            for (int i = 0; i < pad.key.Length; i++)
            {
                if ((padInput & mul) != 0)
                    this.pad.key[i]++;
                else
                    this.pad.key[i] = 0;
                mul *= 2;
            }

            this.input_pad_or_key(ref pad.key[GV.configpad.left], this.CheckStateKey(DX.KEY_INPUT_LEFT));
            this.input_pad_or_key(ref pad.key[GV.configpad.up], this.CheckStateKey(DX.KEY_INPUT_UP));
            this.input_pad_or_key(ref pad.key[GV.configpad.right], this.CheckStateKey(DX.KEY_INPUT_RIGHT));
            this.input_pad_or_key(ref pad.key[GV.configpad.down], this.CheckStateKey(DX.KEY_INPUT_DOWN));
            this.input_pad_or_key(ref pad.key[GV.configpad.shot], this.CheckStateKey(DX.KEY_INPUT_Z));
            this.input_pad_or_key(ref pad.key[GV.configpad.bom], this.CheckStateKey(DX.KEY_INPUT_X));
            this.input_pad_or_key(ref pad.key[GV.configpad.slow], this.CheckStateKey(DX.KEY_INPUT_LSHIFT));
            this.input_pad_or_key(ref pad.key[GV.configpad.start], this.CheckStateKey(DX.KEY_INPUT_ESCAPE));
            this.input_pad_or_key(ref pad.key[GV.configpad.change], this.CheckStateKey(DX.KEY_INPUT_LCONTROL));
        }

        /// <summary>
        /// 渡されたパッドキー番号の入力状態を返します。
        /// </summary>
        /// <param name="handle">パッドキー番号</param>
        /// <returns>入力状態。-1の場合は不正</returns>
        public int ChckStatePad(int handle)
        {
            if (0 <= handle && handle < this.pad.key.Length)
                return this.pad.key[handle];
            else
            {
                Debug.WriteLine("CheckStatePadに渡した値が不正です");
                return -1;
            }
        }
    }
}
