﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// グラフィックスを管理するクラスです。
    /// </summary>
    class GraphManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static GraphManager manager = new GraphManager();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GraphManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static GraphManager GetInstance()
        {
            return manager;
        }
        
        /// <summary>
        /// 背景番号 0の背景を描画します。
        /// </summary>
        private void graph_back_00()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawArea(Setting.FIELD_X, Setting.FIELD_Y, Setting.FIELD_X + Setting.FIELD_MAX_X, Setting.FIELD_Y + Setting.FIELD_MAX_Y); // 描画可能エリアを設定
            DX.DrawGraph(Setting.FIELD_X, GV.count % 700 + Setting.FIELD_Y - 700, imageManager.img_back[0], 0);
            DX.DrawGraph(Setting.FIELD_X, GV.count % 700 + Setting.FIELD_Y, imageManager.img_back[0], 0);
            DX.SetDrawArea(0, 0, 640, 480); // エリアを戻す
        }

        /// <summary>
        /// 背景を描画するメイン処理です。
        /// </summary>
        private void graph_back_main()
        {
            this.graph_back_00();
        }

        /// <summary>
        /// エフェクト群を描画します。
        /// </summary>
        private void graph_effect()
        {
            for (int i = 0; i < GV.effects.length(); i++)
            {
                Effect effect = GV.effects[i];
                if (effect.flag > 0)
                {
                    if (effect.eff == 1) // エフェクトが光エフェクトなら
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, effect.brt);

                    DX.DrawRotaGraphF((float)effect.x + Setting.FIELD_X, (float)effect.y + Setting.FIELD_Y,
                        effect.r, effect.angle, effect.img, 1);

                    if (effect.eff == 1)
                        DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                }
            }
        }

        /// <summary>
        /// 敵キャラクタ群を描画します。
        /// </summary>
        private void graph_enemy()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.enemys.length(); i++)
            {
                Enemy enemy = GV.enemys[i];
                if (enemy.flag == 1)
                    DX.DrawRotaGraphF((float)enemy.x + Setting.FIELD_X, (float)enemy.y + Setting.FIELD_Y, 1.0, 0.0, imageManager.img_enemy[0, enemy.img], 1);
            }
        }

        /// <summary>
        /// ショットを描画します。
        /// </summary>
        private void graph_cshot()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            for (int i = 0; i < GV.cshots.length(); i++)
            {
                CShot cshot = GV.cshots[i];
                if (cshot.flag > 0)
                    DX.DrawRotaGraphF((float)cshot.x + Setting.FIELD_X, (float)cshot.y + Setting.FIELD_Y, 1.0, 0.0, imageManager.img_cshot[cshot.knd], 1);
            }
        }

        /// <summary>
        /// 自キャラクタを描画します。
        /// </summary>
        private void graph_ch()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            if(GV.ch.mutekicnt % 2 == 0)
                DX.DrawRotaGraphF((float)GV.ch.x + Setting.FIELD_X, (float)GV.ch.y + Setting.FIELD_Y, 1.0, 0.0, imageManager.img_ch[0, GV.ch.img], 1);
        }

        /// <summary>
        /// 弾幕を描画します。
        /// </summary>
        private void graph_bullet()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawMode(DX.DX_DRAWMODE_BILINEAR); // 線形補間描画
            for (int i = 0; i < GV.shots.length(); i++) // 弾幕数分ループ
            {
                Shot shot = GV.shots[i];
                if (shot.flag > 0) // 弾幕データがオンなら
                {
                    BulletGroup bullets = shot.bullets;
                    for (int j = 0; j < bullets.length(); j++) // 弾幕が持つの弾の数分ループ
                    {
                        Bullet bullet = bullets[j];
                        if (bullet.flag != 0) // 弾データがオンなら
                        {
                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_ADD, 255);

                            DX.DrawRotaGraphF((float)bullet.x + Setting.FIELD_X, (float)bullet.y + Setting.FIELD_Y,
                                1.0, bullet.angle + Math.PI / 2.0,
                                imageManager.img_bullet[bullet.knd, bullet.col], 1);

                            if (bullet.eff == 1)
                                DX.SetDrawBlendMode(DX.DX_BLENDMODE_NOBLEND, 0);
                        }
                    }
                }
            }
            DX.SetDrawMode(DX.DX_DRAWMODE_NEAREST); // 描画形式を戻す
        }

        /// <summary>
        /// ボードを描画します。
        /// </summary>
        private void graph_board()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.DrawGraph(0, 0, imageManager.img_board[10], 0);
            DX.DrawGraph(0, 16, imageManager.img_board[11], 0);
            DX.DrawGraph(0, 464, imageManager.img_board[12], 0);
            DX.DrawGraph(416, 0, imageManager.img_board[20], 0);
        }

        /// <summary>
        /// 描画します。
        /// </summary>
        public void graph_main()
        {
            this.graph_back_main();
            this.graph_effect();
            this.graph_enemy();
            this.graph_cshot();
            this.graph_ch();
            this.graph_bullet();
            this.graph_board();
        }
    }
}
