﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 弾を表すクラスです。
    /// </summary>
    class Bullet
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 色。
        /// </summary>
        public int col;

        /// <summary>
        /// 状態。
        /// </summary>
        public int state;

        /// <summary>
        /// 少なくとも消さない時間。
        /// </summary>
        public int till;

        /// <summary>
        /// エフェクトの種類。
        /// </summary>
        public int eff;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 角度。
        /// </summary>
        public double angle;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;

        /// <summary>
        /// ベースの角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// 一次記憶スピード。
        /// </summary>
        public double[] rem_spd = new double[1];

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.col = 0;
            this.state = 0;
            this.till = 0;
            this.eff = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.angle = 0.0;
            this.spd = 0.0;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0.0;
            for (int i = 0; i < this.rem_spd.Length; i++)
                this.rem_spd[i] = 0.0;
        }

        /// <summary>
        /// 計算します。
        /// </summary>
        public void calc()
        {
            this.x += Math.Cos(this.angle) * this.spd;
            this.y += Math.Sin(this.angle) * this.spd;
            this.cnt++;

            if (this.x < -50 || Setting.FIELD_MAX_X + 50 < this.x || this.y < -50 || Setting.FIELD_MAX_Y + 50 < this.y) // 画面から外れたら
                if (this.till < this.cnt) // 最低消えない時間より長ければ
                    this.flag = 0; // 消す
        }

        /// <summary>
        /// 自キャラクタに当たったかどうか判定する。
        /// </summary>
        /// <param name="ch">自キャタクタ</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_enemyshot(Ch ch)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                double x = this.x - ch.x; // 敵弾と自キャラクタとの差
                double y = this.y - ch.y;

                // オーバーフロー対策
                if (this.knd >= GV.bullet_infos.length())
                {
                    Debug.WriteLine("out_judge_enemyshot内オーバーフロー");
                    return false;
                }

                // 自キャラクタの当たり判定と敵弾の当たり判定の合計範囲
                double r = Setting.CRANGE + GV.bullet_infos[this.knd].range;

                // 中間を計算する必要があれば
                if (this.spd > r)
                {
                    // 1フレーム前にいた位置を格納する
                    double pre_x = this.x + Math.Cos(this.angle + Math.PI) * this.spd;
                    double pre_y = this.y + Math.Sin(this.angle + Math.PI) * this.spd;
                    double px;
                    double py;

                    for (int j = 0; j < (int)(this.spd / r); j++) // 進んだ分÷当たり判定分ループ
                    {
                        // 自キャラクタと敵弾との差
                        px = pre_x - ch.x;
                        py = pre_y - ch.y;

                        if (px * px + py * py < r * r) // 当たり範囲内なら
                            return true; // 当たり

                        // 調べる範囲を進める
                        pre_x += Math.Cos(this.angle) * r;
                        pre_y += Math.Sin(this.angle) * r;
                    }
                }

                if (x * x + y * y < r * r) // 当たり判定内なら
                    return true; // 当たり
            }

            return false;
        }
        
        /// <summary>
        /// 弾が自キャラクタに当たる処理をします。
        /// </summary>
        /// <returns>自キャラクタが死んだとき true</returns>
        public bool enemyshot_and_ch()
        {
            if (this.out_judge_enemyshot(GV.ch)) // 自キャラクタに弾が接触したら
            {
                this.flag = 0; // 弾をオフ

                // TODO 喰らいボムをここに追加

                if (GV.ch.flag == 0 && GV.ch.mutekicnt == 0) // ステータスが通常で、無敵でなかったら
                {
                    GV.ch.death(); // 自キャラクタの死亡処理

                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[3] = 1;
                    return true;
                }
            }

            return false;
        }
    }
}
