﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 自キャラクタを表すクラスです。
    /// </summary>
    class Ch
    {
        /// <summary>
        ///  フラグ(0=通常、1=喰らいボム受付中、2=画面下から浮き上がってくる)。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// パワー。
        /// </summary>
        public int power;

        /// <summary>
        /// ポイント。
        /// </summary>
        public int point;

        /// <summary>
        /// スコア。
        /// </summary>
        public int score;

        /// <summary>
        /// 残機数。
        /// </summary>
        public int num;

        /// <summary>
        /// 無敵状態とカウント(0ではないとき無敵)。
        /// </summary>
        public int mutekicnt;

        /// <summary>
        /// ショットモード。
        /// </summary>
        public int shot_mode;

        /// <summary>
        /// お金。
        /// </summary>
        public int money;

        /// <summary>
        /// 画像。
        /// </summary>
        public int img;

        /// <summary>
        /// スローかどうか。
        /// </summary>
        public int slow;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// ショットのカウンタ。
        /// </summary>
        public int shot_cnt;

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.power = 0;
            this.point = 0;
            this.score = 0;
            this.num = 0;
            this.mutekicnt = 0;
            this.shot_mode = 0;
            this.money = 0;
            this.img = 0;
            this.slow = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.shot_cnt = 0;
        }

        /// <summary>
        /// ゲームの初期化をします。
        /// </summary>
        public void ini()
        {
            this.x = Setting.FIELD_MAX_X / 2;
            this.y = Setting.FIELD_MAX_Y * 3 / 4;
            this.power = 500;
        }

        /// <summary>
        /// ショットカウントをインクリメントします。
        /// </summary>
        public void increment_shot_cnt()
        {
            this.shot_cnt++;
        }

        /// <summary>
        /// ショットカウントをリセットします。
        /// </summary>
        public void reset_shot_cnt()
        {
            this.shot_cnt = 0;
        }

        /// <summary>
        /// キャラクタ画像をループします。
        /// </summary>
        public void calc_ch()
        {
            if (this.cnt == 0 && this.flag == 2) // 今の瞬間死んだら
            {
                this.x = Setting.FIELD_MAX_X / 2; // 座標をセット
                this.y = Setting.FIELD_MAX_Y + 30;
                this.mutekicnt++; // 無敵状態へ
            }

            if (this.flag == 2) // 死んで浮上中なら
            {
                KeyManager keyManager = KeyManager.GetInstance();
                int push = keyManager.ChckStatePad(GV.configpad.left) + keyManager.ChckStatePad(GV.configpad.right)
                    + keyManager.ChckStatePad(GV.configpad.up) + keyManager.ChckStatePad(GV.configpad.down);

                this.y -= 1.5; // キャラを上に上げる
                if(this.cnt > 60 || (this.y < Setting.FIELD_MAX_Y - 20 && push > 0)) // 1秒以上か、キャラがある程度上にいて何か押されたら
                {
                    this.cnt = 0;
                    this.flag = 0; // キャラステータスを元に戻す
                }
            }

            if (this.mutekicnt > 0) // 無敵カウントが 0じゃなければ
            {
                this.mutekicnt++;
                if (this.mutekicnt > 120) // 2秒以上経ったら
                    this.mutekicnt = 0; // 戻す
            }
            
            this.cnt++; // キャラクタカウントアップ
            this.img = (this.cnt % 24) / 6; // 現在の画像を決定
        }

        /// <summary>
        /// キャラクタの移動制御をします。
        /// </summary>
        public void ch_move()
        {
            int sayu_flag = 0;
            int joge_flag = 0;
            double x;
            double y;
            double mx;
            double my;
            double naname = 1.0;
            double[] move_x = { -4.0, 4.0, 0.0, 0.0 }; // 左右のスピード
            double[] move_y = { 0.0, 0.0, 4.0, -4.0 }; // 上下のスピード
            int[] inputpad = new int[4];

            KeyManager keyManager = KeyManager.GetInstance();
            inputpad[0] = keyManager.ChckStatePad(GV.configpad.left);
            inputpad[1] = keyManager.ChckStatePad(GV.configpad.right);
            inputpad[2] = keyManager.ChckStatePad(GV.configpad.down);
            inputpad[3] = keyManager.ChckStatePad(GV.configpad.up);

            if (keyManager.ChckStatePad(GV.configpad.left) > 0) // 左キーが押されていたら
                this.img += 4 * 2; // 画像を左向きに
            else if (keyManager.ChckStatePad(GV.configpad.right) > 0) // 右キーが押されていたら
                this.img += 4 * 1; // 画像を右向きに

            for (int i = 0; i < 2; i++) // 左右分
                if (inputpad[i] > 0) // 左右どちらかの入力があれば
                    sayu_flag = 1; // 左右入力フラグを立てる
            for (int i = 2; i < 4; i++) // 上下分
                if (inputpad[i] > 0) // 上下どちらかの入力があれば
                    joge_flag = 1; // 上下フラグを立てる

            if (sayu_flag == 1 && joge_flag == 1) // 左右、上下両方の入力があれば斜めだということ
                naname = Math.Sqrt(2.0); // 移動スピードを 1/ルート2に

            for (int i = 0; i < inputpad.Length; i++)
            {
                if (inputpad[i] > 0)// i方向のキーボード、パッドどちらかの入力があれば
                {
                    // 今の X座標をとりあえず x,yに格納
                    x = this.x;
                    y = this.y;

                    // 移動分を mx, myに代入
                    mx = move_x[i]; 
                    my = move_y[i];
                    if (keyManager.ChckStatePad(GV.configpad.slow) > 0) // 低速移動なら
                    {
                        // 移動スピードを 1/3に
                        mx = move_x[i] / 3.0;
                        my = move_y[i] / 3.0;
                    }

                    // 今の座標と移動分を足す
                    x += mx / naname;
                    y += my / naname;

                    if (! (x < 10 || x > Setting.FIELD_MAX_X - 10 || y < 5 || y > Setting.FIELD_MAX_Y - 5)) // 計算結果が移動範囲内なら
                    {
                        // 実際に移動させる
                        this.x = x;
                        this.y = y;
                    }
                }
            }
        }
        
        /// <summary>
        /// 自キャラクタの死亡処理をします。
        /// </summary>
        public void death()
        {
            this.flag = 2;
            this.cnt = 0;
        }
        
        /// <summary>
        /// 自キャラクタを無敵状態にします。
        /// </summary>
        public void setMuteki()
        {
            this.mutekicnt = 1;
        }
    }
}
