﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// エフェクトを表すクラスです。
    /// </summary>
    class Effect
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;
        
        /// <summary>
        /// 色。
        /// </summary>
        public int col;

        /// <summary>
        /// 種類
        /// 0=敵の消滅エフェクト、1=ボムのエフェクト、2=ボムのエフェクト(キャラ)、3=ボムのエフェクト(ライン)。
        /// </summary>
        public int knd;

        /// <summary>
        /// 画像。
        /// </summary>
        public int img;
        
        /// <summary>
        /// エフェクトの種類(1=光エフェクト)。
        /// </summary>
        public int eff;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 半径。
        /// </summary>
        public double r;

        /// <summary>
        /// 角度。
        /// </summary>
        public double ang;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;

        /// <summary>
        /// 不明。
        /// </summary>
        public double mvang;

        /// <summary>
        /// 明るさ。
        /// </summary>
        public double brt;

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.col = 0;
            this.knd = 0;
            this.img = 0;
            this.eff = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.r = 0.0;
            this.ang = 0.0;
            this.spd = 0.0;
            this.mvang = 0.0;
            this.brt = 0.0;
        }
        
        /// <summary>
        /// エフェクトを計算します。
        /// </summary>
        public void calc_effect()
        {
            switch (this.knd) // エフェクトの種類によって分岐
            {
                case 0: // 敵の消滅エフェクト
                    this.cnt++;
                    this.r += 0.08; // エフェクトの大きさを段々大きくする
                    if (this.cnt > 10) // 10カウント以上になったら
                        this.brt -= 25; // 明るさを少なくする
                    if (this.cnt > 20) // 20カウント以上になったら
                        this.flag = 0; // 消す
                    break;
                case 1: // ボムのエフェクト
                    // 座標計算
                    this.x += Math.Cos(this.mvang) * this.spd;
                    this.y += Math.Sin(this.mvang) * this.spd;

                    // スピード計算
                    if (this.cnt < 60)
                        this.spd -= 0.2 + this.cnt * this.cnt / 3000.0;
                    if (this.cnt == 60)
                    {
                        this.spd = 0.0;
                        GV.dn.setDn(); // ドン揺れを設定する。音も鳴らす
                    }

                    // 明るさと大きさを計算
                    this.r += 0.015;
                    if (this.cnt < 51)
                        this.brt += 5.0;
                    if (this.cnt >= 60)
                    {
                        this.r += 0.04;
                        this.brt -= 255 / 30.0;
                    }
                    
                    // カウントアップと消去計算
                    this.cnt++;
                    if (this.cnt >= 90)
                        this.flag = 0;
                    break;
                case 2: // ボムエフェクト(キャラ)
                    // 座標計算
                    this.x += Math.Cos(this.mvang) * this.spd;
                    this.y += Math.Sin(this.mvang) * this.spd;
                    
                    // 明るさ計算
                    if (this.cnt < 51)
                        this.brt += 4.0;
                    if (this.cnt > 130 - 51)
                        this.brt -= 4.0;
                    
                    // カウントアップと消去計算
                    this.cnt++;
                    if (this.cnt >= 130)
                        this.flag = 0;
                    break;
                case 3: // ボムのエフェクト(ライン)
                    // 座標計算
                    this.x += Math.Cos(this.mvang) * this.spd;
                    this.y += Math.Sin(this.mvang) * this.spd;

                    // 明るさ計算
                    if (this.cnt < 51)
                        this.brt += 2.0;
                    if (this.cnt > 130 - 51)
                        this.brt -= 2.0;

                    // カウントアップと消去計算
                    this.cnt++;
                    if (this.cnt >= 130)
                        this.flag = 0;
                    break;
                default:
                    Debug.WriteLine("effect設定エラー(" + this.knd + ")");
                    break;
            }
        }
    }
}
