﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの弾幕を表すクラスです。
    /// </summary>
    class BossShot
    {
        /// <summary>
        /// 不明。
        /// </summary>
        private const int TM000 = 120;

        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;
        
        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// どの敵から発射されたのか。
        /// </summary>
        public Enemy parent_enemy;

        /// <summary>
        /// ベース角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// ベーススピード。
        /// </summary>
        public double[] base_spd = new double[1];

        /// <summary>
        /// 弾群。
        /// </summary>
        public BulletGroup bullets = new BulletGroup(Setting.BOSS_BULLET_MAX);
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.parent_enemy = null;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0;
            for (int i = 0; i < this.base_spd.Length; i++)
                this.base_spd[i] = 0;
            this.bullets.initialize();
        }

        /// <summary>
        /// 空いている弾を返します。
        /// </summary>
        /// <returns>弾。無い場合には null</returns>
        public Bullet search_boss_shot()
        {
            return this.bullets.search_unused_bullet();
        }

        /// <summary>
        /// 自キャラクタとボスとの成す角度を返します。
        /// </summary>
        /// <returns>角度</returns>
        private double bossatan2()
        {
            return Math.Atan2(GV.ch.y - GV.boss.y, GV.ch.x - GV.boss.x);
        }

        /// <summary>
        /// ボスの弾幕を計算します。
        /// </summary>
        public void boss_shot_calc()
        {
            GV.boss.decrease_endtime();
            this.bullets.calc(); // ボスの弾幕の計算は敵の弾幕の計算そのまま

            this.cnt++;
        }

        /// <summary>
        /// 弾幕をセットします。
        /// </summary>
        public void enter_boss_shot()
        {
            this.initialize(); // ショット・弾情報を初期化
            this.flag = 1;
        }

        /// <summary>
        /// 弾幕 000。
        /// </summary>
        private void boss_shot_bulletH000()
        {
            int t = this.cnt % TM000;
            if (t < 60 && t % 10 == 0)
            {
                double angle = this.bossatan2();
                for (int i = 0; i < 30; i++)
                {
                    Bullet bullet = this.search_boss_shot();
                    if (bullet != null)
                    {
                        MusicManager musicManager = MusicManager.GetInstance();
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = angle + Math.PI * 2 / 30 * i;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕を処理します。必要に応じて弾を追加します。
        /// </summary>
        public void do_boss_shot_bullet()
        {
            switch (GV.boss.knd)
            {
                case 0:
                    this.boss_shot_bulletH000();
                    break;
                default:
                    Debug.WriteLine("boss.kndの値が不正(" + GV.boss.knd + ")です");
                    break;
            }
        }
    }
}
