﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 物理計算用のクラスです。
    /// </summary>
    class Phy
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 移動にかける時間。
        /// </summary>
        public int set_t;

        /// <summary>
        /// 水平成分の加速度。
        /// </summary>
        public double ax;
        
        /// <summary>
        /// 水平成分の諸速度。
        /// </summary>
        public double v0x;

        /// <summary>
        /// 垂直成分の加速度。
        /// </summary>
        public double ay;

        /// <summary>
        /// 垂直成分の加速度。
        /// </summary>
        public double v0y;
        
        /// <summary>
        /// ?
        /// </summary>
        public double vx;

        /// <summary>
        /// ?
        /// </summary>
        public double vy;

        /// <summary>
        /// 初期 X座標。
        /// </summary>
        public double prex;

        /// <summary>
        /// 初期 Y座標。
        /// </summary>
        public double prey;
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.set_t = 0;
            this.ax = 0.0;
            this.v0x = 0.0;
            this.ay = 0.0;
            this.v0y = 0.0;
            this.vx = 0.0;
            this.vy = 0.0;
            this.prex = 0.0;
            this.prey = 0.0;
        }
        
        /// <summary>
        /// 物理演算をさせる登録をします(指定時間 tで定位置に戻します)。
        /// </summary>
        /// <param name="boss">ボス</param>
        /// <param name="t">指定時間</param>
        public void input_phy(Boss boss, int t)
        {
            double ymax_x;
            double ymax_y;

            if (t == 0)
                t = 1;

            this.flag = 1; // 登録オン
            this.cnt = 0; // カウンタ初期化
            this.set_t = t; // 移動にかける時間をセット
            ymax_x = boss.x - Setting.BOSS_POS_X; // 移動したい水平距離
            this.v0x = 2 * ymax_x / t; // 水平成分の初速度
            this.ax = 2 * ymax_x / (t * t); // 水平成分の加速度
            this.prex = boss.x; // 初期 X座標
            ymax_y = boss.y - Setting.BOSS_POS_Y; // 移動したい鉛直距離
            this.v0y = 2 * ymax_y / t; // 鉛直成分の初速度
            this.ay = 2 * ymax_y / (t * t); // 鉛直成分の加速度
            this.prey = boss.y; // 初期 Y座標
        }
        
        /// <summary>
        /// カウンタをインクリメントします。
        /// </summary>
        public void increment_counter()
        {
            this.cnt++;
        }

        /// <summary>
        /// フラグをクリアします。
        /// </summary>
        public void clear_flag()
        {
            this.flag = 0;
        }
    }
}
