﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスを表すクラスです。
    /// </summary>
    class Boss
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 種類。中ボスのとき -1、ボスのとき 0以上。
        /// </summary>
        public int knd;

        /// <summary>
        /// 待機時間。
        /// </summary>
        public int wtime;

        /// <summary>
        /// 状態。1=待機中状態、2=弾幕中。
        /// </summary>
        public int state;

        /// <summary>
        /// 残り時間。
        /// </summary>
        public int endtime;

        /// <summary>
        /// 扇を広げるかどうか(0の場合は広げない)。
        /// </summary>
        public int hagoromo;
        
        /// <summary>
        /// 描画フラグ。
        /// </summary>
        public int graph_flag;

        /// <summary>
        /// HP。
        /// </summary>
        public int hp;

        /// <summary>
        /// HP最大値。
        /// </summary>
        public int hp_max;

        /// <summary>
        /// ?
        /// </summary>
        public int[] appear_count = new int[2];

        /// <summary>
        /// 弾幕それぞれの HP。
        /// </summary>
        public int[] set_hp = new int[Setting.DANMAKU_MAX];

        /// <summary>
        /// 弾幕それぞれの背景の種類。
        /// </summary>
        public int[] back_knd = new int[Setting.DANMAKU_MAX];

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 表示用 X座標。
        /// </summary>
        public double dx;
        
        /// <summary>
        /// 表示用 Y座標。
        /// </summary>
        public double dy;

        /// <summary>
        /// 角度。
        /// </summary>
        public double ang;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;

        /// <summary>
        /// 物理演算用のインスタンス。
        /// </summary>
        private Phy phy = new Phy();

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.knd = 0;
            this.wtime = 0;
            this.state = 0;
            this.endtime = 0;
            this.hagoromo = 0;
            this.graph_flag = 0;
            this.hp = 0;
            this.hp_max = 0;
            for (int i = 0; i < this.appear_count.Length; i++)
                this.appear_count[i] = 0;
            for (int i = 0; i < this.set_hp.Length; i++)
                this.set_hp[i] = 0;
            for (int i = 0; i < this.back_knd.Length; i++)
                this.back_knd[i] = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.ang = 0.0;
            this.spd = 0.0;
            this.phy.initialize();
        }

        /// <summary>
        /// 物理的キャラクタ移動計算をします。
        /// </summary>
        private void calc_phy()
        {
            double t = this.phy.cnt;
            this.x = this.phy.prex - ((this.phy.v0x * t) - 0.5 * this.phy.ax * t * t); // 現在いるべき X座標計算
            this.y = this.phy.prey - ((this.phy.v0y * t) - 0.5 * this.phy.ay * t * t); // 現在いるべき Y座標計算
            this.phy.increment_counter();
            if (this.phy.cnt >= this.phy.set_t) // 移動にかける時間分になったら
                this.phy.clear_flag(); // オフ
        }

        /// <summary>
        /// 終了時間をデクリメントし、時間になったら HPをゼロに設定します。
        /// </summary>
        public void decrease_endtime()
        {
            this.endtime--;
            if (this.endtime < 0)
                this.hp = 0;
        }

        /// <summary>
        /// 弾幕をセットし、ボスの情報を設定します。
        /// </summary>
        public void enter_boss_shot()
        {
            GV.boss_shot.enter_boss_shot(); // 弾幕を設定

            this.wtime = 0; // 待機時間を 0
            this.state = 2; // 弾幕中状態にする
            this.hp = this.set_hp[this.knd]; // HPを設定
            this.hp_max = this.hp;
        }

        /// <summary>
        /// ボスをセットします。
        /// </summary>
        /// <param name="num">番号(0のときは中ボス)</param>
        public void enter_boss(int num)
        {
            if (num == 0) // 中ボス開始時は
            {
                GV.enemys.initialize(); // 雑魚敵を消す
                GV.shots.initialize(); // 弾幕を消す

                this.x = Setting.FMX / 2; // ボスの初期座標
                this.y = -30.0;
                this.knd = -1; // 弾幕の種類
            }

            this.flag = 1;
            this.hagoromo = 0; // 扇を広げるかどうかのフラグ
            this.endtime = 99 * 60; // 残り時間
            this.state = 1; // 待機中状態にする
            this.cnt = 0;
            this.graph_flag = 0; // 描画フラグを戻す
            this.knd++;
            this.wtime = 0; // 待機時間を初期化

            GV.boss_shot.initialize(); // ボスの弾幕情報を初期化
            this.phy.input_phy(this, 60); // 60カウントかけて物理的演算で定位置に戻す
        }

        /// <summary>
        /// ボスの待機処理をします。
        /// </summary>
        public void waitandenter()
        {
            int t = 140;
            this.wtime++;
            if (this.wtime > t) // 140カウント待機したら弾幕セット
                this.enter_boss_shot();
        }

        /// <summary>
        /// ボスの弾幕メインを実行します。
        /// </summary>
        public void boss_shot_main()
        {
            if (GV.stage_count == this.appear_count[0] && this.flag == 0) // 開始時間なら
                this.enter_boss(0); // 開始
            if (this.flag == 0) // ボスが登録されていなければ
                return; // 戻る

            this.calc_boss(); // 表示用座標を計算する

            if (this.phy.flag == 1) // 物理演算移動オンなら
                this.calc_phy(); // 物理演算を実行する
            if (this.state == 2 && (this.hp <= 0 || this.endtime <= 0)) // 弾幕中で、体力が無くなったか時間が経過したら
                this.enter_boss(1); // 次の弾幕を登録

            if (this.state == 1) // 弾幕中の待機時間なら
                this.waitandenter();

            if (this.state == 2) // 弾幕中なら
            {
                GV.boss_shot.do_boss_shot_bullet(); // 弾幕群を処理する(必要に応じて弾を追加する)
                GV.boss_shot.boss_shot_calc(); // 弾幕を計算する
            }

            this.cnt++;
        }

        /// <summary>
        /// ボスに power食らわします。
        /// </summary>
        /// <param name="power">パワー</param>
        public void hit_boss(int power)
        {
            this.hp -= power; // 自キャラクタのショットが持つパワー分 HPを減らす
        }

        /// <summary>
        /// 表示用座標を計算します。
        /// </summary>
        private void calc_boss()
        {
            this.dx = this.x;
            this.dy = this.y + Math.Sin(Math.PI * 2 / 130 * (GV.count % 130)) * 10;
        }

        /// <summary>
        /// 今いる位置から dist離れた位置に tカウントで移動します。
        /// 通常は 0を返し、1000回試して駄目なら -1を返します。
        /// </summary>
        /// <param name="x1">移動可能範囲 X1座標</param>
        /// <param name="y1">移動可能範囲 Y1座標</param>
        /// <param name="x2">移動可能範囲 X2座標</param>
        /// <param name="y2">移動可能範囲 Y2座標</param>
        /// <param name="dist">距離</param>
        /// <param name="t">カウント</param>
        /// <returns>0の場合は正常終了、-1の場合は異常終了</returns>
        public int move_boss_pos(double x1, double y1, double x2, double y2, double dist, int t)
        {
            double x;
            double y;
            double angle;
            for (int i = 0; i < 1000; i++)
            {
                x = GV.boss.x; // 今のボスの位置をセット
                y = GV.boss.y;
                angle = MyMath.rang(Math.PI); // 適当に向かう方向を決める
                x += Math.Cos(angle) * dist; // 移動させる
                y += Math.Sin(angle) * dist;
                if (x1 <= x && x <= x2 && y1 <= y && y <= y2) // その点が移動可能範囲なら
                {
                    this.phy.input_phy_pos(GV.boss, x, y, t);
                    return 0;
                }
            }

            return -1; // 異常終了
        }

        /// <summary>
        /// ゲームの初期化をします。
        /// </summary>
        public void ini()
        {
            this.appear_count[0] = 500; // 中ボスが出現する時刻
            for (int i = 0; i < this.set_hp.Length; i++)
                this.set_hp[i] = 10000;
            this.hp_max = 10000;
            for (int i = 0; i < this.back_knd.Length; i ++)
                this.back_knd[i] = 1;
        }
    }
}
