﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの弾幕を表すクラスです。
    /// </summary>
    class BossShot
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;
        
        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// どの敵から発射されたのか。
        /// </summary>
        public Enemy parent_enemy;

        /// <summary>
        /// ベース角度。
        /// </summary>
        public double[] base_angle = new double[1];

        /// <summary>
        /// ベーススピード。
        /// </summary>
        public double[] base_spd = new double[1];

        /// <summary>
        /// 弾群。
        /// </summary>
        public BulletGroup bullets = new BulletGroup(Setting.BOSS_BULLET_MAX);
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.knd = 0;
            this.cnt = 0;
            this.parent_enemy = null;
            for (int i = 0; i < this.base_angle.Length; i++)
                this.base_angle[i] = 0;
            for (int i = 0; i < this.base_spd.Length; i++)
                this.base_spd[i] = 0;
            this.bullets.initialize();
        }

        /// <summary>
        /// 空いている弾を返します。
        /// </summary>
        /// <returns>弾。無い場合には null</returns>
        public Bullet search_boss_shot()
        {
            return this.bullets.search_unused_bullet();
        }

        /// <summary>
        /// 自キャラクタとボスとの成す角度を返します。
        /// </summary>
        /// <returns>角度</returns>
        private double bossatan2()
        {
            return Math.Atan2(GV.ch.y - GV.boss.y, GV.ch.x - GV.boss.x);
        }

        /// <summary>
        /// ボスの弾幕を計算します。
        /// </summary>
        public void boss_shot_calc()
        {
            GV.boss.decrease_endtime();
            this.bullets.calc(); // ボスの弾幕の計算は敵の弾幕の計算そのまま

            this.cnt++;
        }

        /// <summary>
        /// 弾幕をセットします。
        /// </summary>
        public void enter_boss_shot()
        {
            this.initialize(); // ショット・弾情報を初期化
            this.flag = 1;
        }

        /// <summary>
        /// ボスショットと自キャラクタとの処理をします。
        /// </summary>
        public void bossshot_and_ch()
        {
            if (this.flag > 0) // ショットが登録されていれば
                this.bullets.enemyshot_and_ch();
        }

        /// <summary>
        /// 弾幕 000。
        /// </summary>
        private void boss_shot_bulletH000()
        {
            const int TM000 = 120;

            int t = this.cnt % TM000;
            if (t < 60 && t % 10 == 0)
            {
                double angle = this.bossatan2();
                for (int i = 0; i < 30; i++)
                {
                    Bullet bullet = this.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = angle + Math.PI * 2 / 30 * i;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 001用のターン番号記憶変数。
        /// </summary>
        private int cnum001;

        /// <summary>
        /// 弾幕 001。サイレントセレナ。
        /// </summary>
        private void boss_shot_bulletH001()
        {
            const int TM001 = 60;

            int t = this.cnt % TM001;
            if (this.cnt == 0) // 最初の初期化
                this.cnum001 = 0;
            if (t == 0) // 1ターン目の最初の初期化
            {
                this.base_angle[0] = this.bossatan2(); // 自キャラクタとボスの角度
                if (this.cnum001 % 4 == 3) // 4ターンに 1回移動
                    GV.boss.move_boss_pos(40, 30, Setting.FMX - 40, 120, 60.0, 60);
            }

            if (t == TM001 / 2 - 1) // 半分からは
                this.base_angle[0] += 2 * Math.PI / 20 / 2; // 自キャラクタ狙いを外す

            // 円形弾群の登録
            if (t % (TM001 / 10) == 0) // 1ターンに 10回
            {
                double angle = this.bossatan2(); // 自キャラクタとボスの角度
                for (int i = 0; i < 20; i++)
                {
                    Bullet bullet = this.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 1; // 原作では 4に設定されているが、プレイ動画の色から判断すると 1。
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = this.base_angle[0] + 2 * Math.PI / 20 * i; // ベース角度から 20個回転して発射
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 2.7; // スピード

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }

            // 4カウントに 1回、下に落ちる弾を登録
            if (t % 4 == 0)
            {
                Bullet bullet = this.search_boss_shot();
                if (bullet != null)
                {
                    bullet.col = 0;
                    bullet.x = MyMath.GetRand(Setting.FMX);
                    bullet.y = MyMath.GetRand(200);
                    bullet.knd = 8;
                    bullet.angle = Math.PI / 2; // 真下の角度
                    bullet.flag = 1;
                    bullet.cnt = 0;
                    bullet.spd = 1 + MyMath.rang(0.5);

                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            if (t == TM001 - 1)
                this.cnum001++;
        }

        /// <summary>
        /// 弾幕 002。パーフェクトシリーズ。
        /// </summary>
        private void boss_shot_bulletH002()
        {
            const int TM002 = 650;
            
            int t = this.cnt % TM002;
            if(t == 0 || t == 210)
            {
                // 40 < x < FMX - 40、50 < y < 150 の範囲で 100離れた位置に 80カウントで移動する
                GV.boss.move_boss_pos(40.0, 50.0, Setting.FMX, 150, 100, 80);
            }

            // 最初のランダム発射
            if(t < 180)
            {
                for(int i = 0; i < 2; i++) // 1カウントに 2回発射
                {
                    Bullet bullet = this.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = MyMath.GetRand(6); // 7種類の色をランダムに
                        bullet.x = GV.boss.x; // 発射初期座標はボスの位置
                        bullet.y = GV.boss.y;
                        bullet.knd = 7; // 弾の種類
                        bullet.angle = MyMath.rang(2 * Math.PI / 20) + 2 * Math.PI / 10 * t;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.2 + MyMath.rang(2.1);
                        bullet.state = 0; // 弾のステータス
                        bullet.kaiten = 1; // 弾を回転するかどうかのフラグ
                    }
                }
                if (t % 10 == 0)
                {
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            // 自キャラクタの位置に依存する 8方向発射
            if (210 < t && t < 270 && t % 3 == 0)
            {
                double angle = this.bossatan2();
                for (int i = 0; i < 8; i++)
                {
                    Bullet bullet = this.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 7;
                        // 自キャラクタとボスとの成す角を基準に 8方向に発射する
                        bullet.angle = angle - Math.PI / 2 * 0.8 + Math.PI * 0.8 / 7 * i + MyMath.rang(Math.PI / 180);
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0 + MyMath.rang(0.3);
                        bullet.state = 2; // 弾のステータス
                        bullet.kaiten = 1;
                    }
                }
                if (t % 10 == 0)
                {
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            for (int i = 0; i < this.bullets.length(); i++)
            {
                Bullet bullet = this.bullets[i];
                if (bullet.flag > 0)
                {
                    // tが 190の時にすべてストップさせ、白くし、カウントリセット
                    if (bullet.state == 0)
                    {
                        if (t == 190)
                        {
                            bullet.kaiten = 0; // 回転を止める
                            bullet.spd = 0.0;
                            bullet.col = 9; // 本来は 9を設定したいのだけど暫定的に 8を設定
                            bullet.cnt = 0; // カウントリセット
                            bullet.state = 1; // ステータスを 1に
                        }
                    }
                    if (bullet.state == 1)
                    {
                        if (bullet.cnt == 200)
                        {
                            bullet.angle = MyMath.rang(Math.PI); // 全方向ランダム
                            bullet.kaiten = 1; // 回転するフラグを立てる
                        }
                        if (bullet.cnt > 200)
                            bullet.spd += 0.01; // 段々加速
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕を処理します。必要に応じて弾を追加します。
        /// </summary>
        public void do_boss_shot_bullet()
        {
            switch (GV.boss.knd)
            {
                case 0:
                    this.boss_shot_bulletH002();
                    break;
                case 1:
                    this.boss_shot_bulletH000();
                    break;
                case 2:
                    this.boss_shot_bulletH000();
                    break;
                case 3:
                    this.boss_shot_bulletH000();
                    break;
                case 4:
                    this.boss_shot_bulletH000();
                    break;
                case 5:
                    this.boss_shot_bulletH000();
                    break;
                case 6:
                    this.boss_shot_bulletH000();
                    break;
                default:
                    Debug.WriteLine("boss.kndの値が不正(" + GV.boss.knd + ")です");
                    break;
            }
        }
    }
}
