﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 独自の関数ライブラリです。
    /// </summary>
    class MyMath
    {
        /// <summary>
        /// ランダム。
        /// </summary>
        private static Random random = new Random();

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private MyMath()
        {
            ; // 何もしない
        }

        /// <summary>
        /// 渡された -ang～angまでのランダムな角度を返します。
        /// </summary>
        /// <param name="ang">角度</param>
        /// <returns> -ang～angまでのランダムな角度</returns>
        public static double rang(double ang)
        {
            return -ang + ang * 2 * DX.GetRand(10000) / 10000.0;
        }

        /// <summary>
        /// 0から nのランダムな数を返します。
        /// </summary>
        /// <param name="n">n</param>
        /// <returns>0から nまでの数</returns>
        public static int GetRand(int n)
        {
            return DX.GetRand(n);
        }

        /// <summary>
        /// 渡された -ang～angまでのランダムな角度を返します。
        /// </summary>
        /// <param name="ang">角度</param>
        /// <returns> -ang～angまでのランダムな角度</returns>
        public static double rang_old(double ang)
        {
            return -ang + ang * 2 * random.Next(0, 10001) / 10000.0;
        }

        /// <summary>
        /// 0から nのランダムな数を返します。
        /// </summary>
        /// <param name="n">n</param>
        /// <returns>0から nまでの数</returns>
        public static int GetRand_old(int n)
        {
            return random.Next(0, n + 1);
        }

        /// <summary>
        /// 当たり判定を返します。
        /// </summary>
        /// <param name="x1">弾の X座標</param>
        /// <param name="y1">弾 Y座標</param>
        /// <param name="x2">物体の X座標</param>
        /// <param name="y2">物体の Y座標</param>
        /// <param name="range1">ショットの範囲</param>
        /// <param name="range2">物体の範囲</param>
        /// <param name="spd1">ショットのスピード</param>
        /// <param name="angle1">ショットの角度</param>
        /// <returns>接触しているなら true</returns>
        public static bool out_judge(double x1, double y1, double x2, double y2, double range1, double range2, double spd1, double angle1)
        {
            double x = x1 - x2; // 弾と物体との差
            double y = y1 - y2;

            // 弾の当たり判定と物体の当たり判定の合計範囲
            double r = range1 + range2;

            // 中間を計算する必要があれば
            if (spd1 > r)
            {
                // 1フレーム前にいた位置を格納する
                double pre_x = x1 + Math.Cos(angle1 + Math.PI) * spd1;
                double pre_y = y1 + Math.Sin(angle1 + Math.PI) * spd1;
                double px;
                double py;

                for (int j = 0; j < (int)(spd1 / r); j++) // 進んだ分÷当たり判定分ループ
                {
                    // 自キャラクタと敵弾との差
                    px = pre_x - x2;
                    py = pre_y - y2;

                    if (px * px + py * py < r * r) // 当たり範囲内なら
                        return true; // 当たり

                    // 調べる範囲を進める
                    pre_x += Math.Cos(angle1) * r;
                    pre_y += Math.Sin(angle1) * r;
                }
            }

            if (x * x + y * y < r * r) // 当たり判定内なら
                return true; // 当たり

            return false;
        }
    }
}
