﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// ボスの弾幕の処理パターンを表すクラスです。
    /// </summary>
    class BossShotPattern
    {
        /// <summary>
        /// 自キャラクタとボスとの成す角度を返します。
        /// </summary>
        /// <returns>角度</returns>
        private static double bossatan2()
        {
            return Math.Atan2(GV.ch.y - GV.boss.y, GV.ch.x - GV.boss.x);
        }

        /// <summary>
        /// 指定座標と弾との成す角度を返します。
        /// </summary>
        /// <param name="bullet">弾</param>
        /// <param name="x">X座標</param>
        /// <param name="y">Y座標</param>
        /// <returns></returns>
        private static double bossatan3(Bullet bullet, double x, double y)
        {
            return Math.Atan2(y - bullet.y, x - bullet.x);
        }

        /// <summary>
        /// 弾幕 000。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH000(BossShot boss_shot)
        {
            const int TM000 = 120;

            int t = boss_shot.cnt % TM000;
            if (t < 60 && t % 10 == 0)
            {
                double angle = bossatan2();
                for (int i = 0; i < 30; i++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = angle + MyMath.PI2 / 30 * i;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 001用フィールド。ターン番号を記憶。
        /// </summary>
        private static int cnum001;

        /// <summary>
        /// 弾幕 001。サイレントセレナ。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH001(BossShot boss_shot)
        {
            const int TM001 = 60;

            int t = boss_shot.cnt % TM001;
            int t2 = boss_shot.cnt;

            if (t2 == 0) // 最初の初期化
                cnum001 = 0;
            if (t == 0) // 1ターン目の最初の初期化
            {
                boss_shot.base_angle[0] = bossatan2(); // 自キャラクタとボスの角度
                if (cnum001 % 4 == 3) // 4ターンに 1回移動
                    GV.boss.move_boss_pos(40, 30, Setting.FMX - 40, 120, 60.0, 60);
            }

            if (t == TM001 / 2 - 1) // 半分からは
                boss_shot.base_angle[0] += MyMath.PI2 / 20 / 2; // 自キャラクタ狙いを外す

            // 円形弾群の登録
            if (t % (TM001 / 10) == 0) // 1ターンに 10回
            {
                double angle = bossatan2(); // 自キャラクタとボスの角度
                for (int i = 0; i < 20; i++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 1; // 原作では 4に設定されているが、プレイ動画の色から判断すると 1。
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 8;
                        bullet.angle = boss_shot.base_angle[0] + MyMath.PI2 / 20 * i; // ベース角度から 20個回転して発射
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 2.7; // スピード

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }

            // 4カウントに 1回、下に落ちる弾を登録
            if (t % 4 == 0)
            {
                Bullet bullet = boss_shot.search_boss_shot();
                if (bullet != null)
                {
                    bullet.col = 0;
                    bullet.x = MyMath.GetRand(Setting.FMX);
                    bullet.y = MyMath.GetRand(200);
                    bullet.knd = 8;
                    bullet.angle = Math.PI / 2; // 真下の角度
                    bullet.flag = 1;
                    bullet.cnt = 0;
                    bullet.spd = 1 + MyMath.rang(0.5);

                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            if (t == TM001 - 1)
                cnum001++;
        }

        /// <summary>
        /// 弾幕 002。パーフェクトシリーズ。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH002(BossShot boss_shot)
        {
            const int TM002 = 650;

            int t = boss_shot.cnt % TM002;
            if (t == 0 || t == 210)
            {
                // 40 < x < FMX - 40、50 < y < 150 の範囲で 100離れた位置に 80カウントで移動する
                GV.boss.move_boss_pos(40, 50, Setting.FMX, 150, 100, 80);
            }

            // 最初のランダム発射
            if (t < 180)
            {
                for (int i = 0; i < 2; i++) // 1カウントに 2回発射
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = MyMath.GetRand(6); // 7種類の色をランダムに
                        bullet.x = GV.boss.x; // 発射初期座標はボスの位置
                        bullet.y = GV.boss.y;
                        bullet.knd = 7; // 弾の種類
                        bullet.angle = MyMath.rang(MyMath.PI2 / 20) + MyMath.PI2 / 10 * t;
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.2 + MyMath.rang(2.1);
                        bullet.state = 0; // 弾のステータス
                        bullet.kaiten = 1; // 弾を回転するかどうかのフラグ
                    }
                }
                if (t % 10 == 0)
                {
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            // 自キャラクタの位置に依存する 8方向発射
            if (210 < t && t < 270 && t % 3 == 0)
            {
                double angle = bossatan2();
                for (int i = 0; i < 8; i++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.col = 0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.knd = 7;
                        // 自キャラクタとボスとの成す角を基準に 8方向に発射する
                        bullet.angle = angle - Math.PI / 2 * 0.8 + Math.PI * 0.8 / 7 * i + MyMath.rang(Math.PI / 180);
                        bullet.flag = 1;
                        bullet.cnt = 0;
                        bullet.spd = 3.0 + MyMath.rang(0.3);
                        bullet.state = 2; // 弾のステータス
                        bullet.kaiten = 1;
                    }
                }
                if (t % 10 == 0)
                {
                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[0] = 1;
                }
            }

            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    // tが 190の時にすべてストップさせ、白くし、カウントリセット
                    if (bullet.state == 0)
                    {
                        if (t == 190)
                        {
                            bullet.kaiten = 0; // 回転を止める
                            bullet.spd = 0.0;
                            bullet.col = 9; // 本来は 9を設定したいのだけど暫定的に 8を設定
                            bullet.cnt = 0; // カウントリセット
                            bullet.state = 1; // ステータスを 1に
                        }
                    }
                    if (bullet.state == 1)
                    {
                        if (bullet.cnt == 200)
                        {
                            bullet.angle = MyMath.rang(Math.PI); // 全方向ランダム
                            bullet.kaiten = 1; // 回転するフラグを立てる
                        }
                        if (bullet.cnt > 200)
                            bullet.spd += 0.01; // 段々加速
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 003用フィールド。tcnt。
        /// </summary>
        private static int tcnt003;

        /// <summary>
        /// 弾幕 003用フィールド。cnt。
        /// </summary>
        private static int cnt003;

        /// <summary>
        /// 弾幕 003用フィールド。cnum;
        /// </summary>
        private static int cnum003;

        /// <summary>
        /// 弾幕 003。恋の迷路。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH003(BossShot boss_shot)
        {
            const int TM003 = 600;
            const int DF003 = 20;

            int t = boss_shot.cnt % TM003;
            int t2 = boss_shot.cnt;

            bool cnum_flag = cnum003 % 2 != 0; // 原作版と異なり 2の剰余を取っています

            if (t2 == 0)
            {
                GV.boss.phy.input_phy_pos(GV.boss, Setting.FMX / 2, Setting.FMY / 2, 50);
                cnum003 = 0;
            }
            if (t == 0)
            {
                boss_shot.base_angle[0] = bossatan2();
                cnt003 = 0;
                tcnt003 = 2;
            }
            if (t < 540 && t % 3 == 0)
            {
                double angle = bossatan2();
                if (tcnt003 - 2 == cnt003 || tcnt003 - 1 == cnt003)
                {
                    if (tcnt003 - 1 == cnt003)
                    {
                        boss_shot.base_angle[1] = boss_shot.base_angle[0] + MyMath.PI2 / DF003 * cnt003 * (cnum_flag ? -1 : 1)
                            - MyMath.PI2 / (DF003 * 6) * 3;
                        tcnt003 += DF003 - 2;
                    }
                }
                else
                {
                    for (int i = 0; i < 6; i++)
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.col = cnum_flag ? 5 : 1; // 原作版と色を変えています
                            bullet.x = GV.boss.x;
                            bullet.y = GV.boss.y;
                            bullet.knd = 8;
                            bullet.angle = boss_shot.base_angle[0]
                                + MyMath.PI2 / DF003 * cnt003 * (cnum_flag ? -1 : 1)
                                + MyMath.PI2 / (DF003 * 6) * i * (cnum_flag ? -1 : 1);
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            bullet.spd = 2.0;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                    }
                }
                cnt003++;
            }
            if (40 < t && t < 540 && t % 30 == 0)
            {
                double angle;
                for (int j = 0; j < 3; j++)
                {
                    angle = boss_shot.base_angle[1] - MyMath.PI2 / 36 * 4;
                    for (int i = 0; i < 27; i++)
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.col = cnum_flag ? 6 : 0;
                            bullet.x = GV.boss.x;
                            bullet.y = GV.boss.y;
                            bullet.knd = 7;
                            bullet.angle = angle;
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            bullet.spd = 4.0 - 1.6 / 3 * j;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                        angle -= MyMath.PI2 / 36;
                    }
                }
            }
            if (t == TM003 - 1)
                cnum003++;
        }

        /// <summary>
        /// 弾幕 004用フィールド。tm。
        /// </summary>
        private static int tm004;

        /// <summary>
        /// 弾幕 004。ケロちゃん風雨に負けず。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH004(BossShot boss_shot)
        {
            const int TM004 = 200;

            int t = boss_shot.cnt % TM004;
            int t2 = boss_shot.cnt;

            if (t == 0) // 周期の最初に tmをセット
                tm004 = 190 + (int)MyMath.rang(30);

            double angle = Math.PI * 1.5 + Math.PI / 6 * Math.Sin(MyMath.PI2 / tm004 * t2);

            // 4カウントに 1回上に 8way弾を撃つ
            if (t2 % 4 == 0)
            {
                for (int n = 0; n < 8; n++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        bullet.flag = 1;
                        bullet.state = 0;
                        bullet.cnt = 0;
                        bullet.knd = 4;
                        bullet.col = 0;
                        bullet.angle = 0.0;
                        bullet.eff_detail = 1;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.vx = Math.Cos(angle - Math.PI / 8 * 4 + Math.PI / 8 * n + Math.PI / 16) * 3;
                        bullet.vy = Math.Sin(angle - Math.PI / 8 * 4 + Math.PI / 8 * n + Math.PI / 16) * 3;
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[0] = 1;
            }
            if (t % 1 == 0 && t2 > 80)
            {
                int num = 1;
                if (t % 2 == 0)
                    num = 2;
                for (int n = 0; n < num; n++)
                {
                    Bullet bullet = boss_shot.search_boss_shot();
                    if (bullet != null)
                    {
                        angle = Math.PI * 1.5 - Math.PI / 2 + Math.PI / 12 * (t2 % 13) + MyMath.rang(Math.PI / 15);
                        bullet.flag = 1;
                        bullet.state = 1;
                        bullet.cnt = 0;
                        bullet.knd = 8;
                        bullet.eff_detail = 0;
                        bullet.col = 1; // 原作版ソースでは 4のところを 1
                        bullet.angle = 0.0;
                        bullet.x = GV.boss.x;
                        bullet.y = GV.boss.y;
                        bullet.vx = Math.Cos(angle) * 1.4 * 1.2;
                        bullet.vy = Math.Sin(angle) * 1.4;
                    }
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[7] = 1;
            }
            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet != null)
                {
                    if (bullet.state == 0)
                    {
                        if (bullet.cnt < 150)
                            bullet.vy += 0.03;
                        bullet.x += bullet.vx;
                        bullet.y += bullet.vy;
                    }
                    if (bullet.state == 1)
                    {
                        if (bullet.cnt < 160)
                            bullet.vy += 0.03;
                        bullet.x += bullet.vx;
                        bullet.y += bullet.vy;
                        bullet.angle = Math.Atan2(bullet.vy, bullet.vx);
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 005用フィールド。sst。
        /// </summary>
        private static int sst005;

        /// <summary>
        /// 弾幕 005用フィールド。bnum。
        /// </summary>
        private static int bnum005;

        /// <summary>
        /// 弾幕 005用フィールド。sx。
        /// </summary>
        private static double sx005;

        /// <summary>
        /// 弾幕 005用フィールド。sy。
        /// </summary>
        private static double sy005;

        /// <summary>
        /// 弾幕 005用フィールド。sangle。
        /// </summary>
        private static double sangle005;
        
        /// <summary>
        /// 弾幕 005。早苗風☆弾。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH005(BossShot boss_shot)
        {
            const int TM005 = 820;
            const double RANGE005 = 100.0;
            const int LEM005 = 4;

            int t = boss_shot.cnt % TM005;
            int t2 = boss_shot.cnt;

            if (t == 0) // 1周期の最初なら
            {
                sst005 = 0;
                sx005 = GV.boss.x;
                sy005 = GV.boss.y - RANGE005;
                sangle005 = Math.PI / 5 / 2 + Math.PI / 2;
            }

            // 星を描く
            if (sst005 <= 4)
            {
                for (int i = 0; i < 2; i++)
                {
                    sx005 += Math.Cos(sangle005) * LEM005; // 座標を計算
                    sy005 += Math.Sin(sangle005) * LEM005;

                    // 円と交わったら
                    if ((sx005 - GV.boss.x) * (sx005 - GV.boss.x) + (sy005 - GV.boss.y) * (sy005 - GV.boss.y) > RANGE005 * RANGE005)
                    {
                        sangle005 -= Math.PI - Math.PI / 5; // 方向転換
                        sst005++; // ステータス変換
                        if (sst005 == 5) // ステータスが 5なら
                            break; // 終わる
                    }
                    for (int j = 0; j < 5; j++) // 星を 5つ描く
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.flag = 1;
                            bullet.state = j;
                            bullet.cnt = 0;
                            bullet.spd = 0.0;
                            bullet.knd = 10;
                            bullet.col = j;
                            bullet.angle = -Math.PI / 2 + MyMath.PI2 / 5 * j;
                            bullet.x = sx005;
                            bullet.y = sy005;
                            bullet.vx = Math.Cos(sangle005) * 1.4 * 1.2;
                            bullet.vy = Math.Sin(sangle005) * 1.4;
                            bullet.base_angle[0] = sangle005 - Math.PI + Math.PI / 20 * bnum005;
                        }
                    }
                    bnum005++;
                }
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[0] = 1;
            }

            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    int cnt = bullet.cnt;

                    if (bullet.state < 10) // ステータスが 10未満なら
                    {
                        if (t == 150) // 150なら星を 5方向に発射
                        {
                            bullet.spd = 4.0; // スピード
                            bullet.cnt = 0; // カウンタリセット
                            bullet.state += 10; // ステータス番号変更
                        }
                    }
                    else if (bullet.state < 20) // ステータスが 10～19なら
                    {
                        if (cnt <= 80) // 減速
                            bullet.spd -= 0.05;
                        if (cnt == 100) // セットした基準角度から綺麗な曲線を描く
                            bullet.angle = bullet.base_angle[0];
                        if (cnt >= 100 && cnt < 160) // 加速
                            bullet.spd += 0.015;
                    }
                }
            }
        }

        /// <summary>
        /// 弾幕 006用フィールド。num。
        /// </summary>
        private static int num006;

        /// <summary>
        /// 弾幕 006。そして誰もいなくなるか? 風弾幕。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH006(BossShot boss_shot)
        {
            const int TM006 = 300;
            const int DNUMX = 15; // 横一列に置く弾の数

            int t = boss_shot.cnt % TM006;
            int t2 = boss_shot.cnt;
            double xlen = (double)Setting.FMX / DNUMX; // 画面に DNUMX個並べるには xlen感覚で並べれば良い
            int dnumy = Setting.FMY / (int)xlen; // xlenずつ並べると鉛直方向にいくつはいるか

            if (dnumy == 0)
            {
                Debug.WriteLine("boss_shot_bulletH006で 0割り");
                return;
            }

            double ylen = (double)Setting.FMY / dnumy; // ylenずつ置けば良い

            if (t2 == 0)
                num006 = 0;
            if (t == 0)
            {
                double x = 0.0;
                double y = 0.0;
                double angle = 0.0;
                int cnum = 0;

                for (int j = 0; j < 4; j++) // 画面左上端から、右→下→左→上の 4方向移動させる
                {
                    int to = (j % 2 != 0) ? dnumy : DNUMX;
                    for (int i = 0; i <= to; i++) // 水平方向、垂直方向では置く個数が違うので to回置く
                    {
                        Bullet bullet = boss_shot.search_boss_shot();
                        if (bullet != null)
                        {
                            bullet.x = x; // 座標
                            bullet.y = y;
                            bullet.knd = 4; // 種類
                            bullet.flag = 1;
                            bullet.cnt = 0;
                            switch (num006)
                            {
                                case 0: // 発射種類 1
                                    bullet.col = 0; // 弾の色
                                    bullet.angle = bossatan3(bullet, Setting.FMX / 2, Setting.FMY / 2); // 角度
                                    bullet.spd = 1.3; // スピード
                                    bullet.state = 0; // ステータス
                                    break;
                                case 1: // 発射種類 2
                                    bullet.col = 3; // 弾の色
                                    bullet.angle = bossatan3(bullet, Setting.FMX / 2, Setting.FMY / 2); // 角度
                                    bullet.spd = 1.4
                                        + (((j % 2 != 0) ? -1 : 1) * ((Math.Cos(MyMath.PI2 / to * i - Math.PI) + 1) / 2)) * 0.4; // スピード
                                    bullet.state = 1; // ステータス
                                    break;
                                case 2: // 発射種類 3
                                    bullet.col = 6; // 弾の色
                                    bullet.angle = bossatan3(bullet, Setting.FMX / 2, Setting.FMY / 2); // 角度
                                    bullet.spd = 1.3; // スピード
                                    bullet.state = 2; // ステータス
                                    bullet.base_angle[0] = Math.PI / 1000 * ((j % 2 != 0) ? -1 : 1) * ((Math.Cos(MyMath.PI2 / to * i - Math.PI) + 1) / 2); // 加算する角度
                                    break;
                            }
                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;

                            cnum++;
                        }
                        x += Math.Cos(angle) * xlen;
                        y += Math.Sin(angle) * xlen;
                    }
                    angle += Math.PI / 2;
                }
            }

            for (int i = 0; i < boss_shot.bullets.length(); i++)
            {
                Bullet bullet = boss_shot.bullets[i];
                if (bullet.flag > 0)
                {
                    int cnt = bullet.cnt;
                    switch (bullet.state)
                    {
                        case 2: // 発射種類 3なら
                            bullet.angle += bullet.base_angle[0]; // 角度を計算
                            break;
                        default:
                            break;
                    }
                }
            }

            if (t == TM006 - 1)
            {
                num006++;
                num006 = num006 % 3;
            }
        }

        /// <summary>
        /// 弾幕 007用フィールド。num。
        /// </summary>
        private static int num007;

        /// <summary>
        /// 弾幕 007。そして誰もいなくなるか? 風弾幕。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        private static void boss_shot_bulletH007(BossShot boss_shot)
        {
            const int TM007 = 300;
            const int DNUMX = 9; // 横一列に置く弾の数

            int t = boss_shot.cnt % TM007;
            int t2 = boss_shot.cnt;
            double xlen = (double)Setting.FMX / DNUMX; // 画面に DNUMX個並べるには xlen感覚で並べれば良い
            int dnumy = Setting.FMY / (int)xlen; // xlenずつ並べると鉛直方向にいくつはいるか

            if (dnumy == 0)
            {
                Debug.WriteLine("boss_shot_bulletH007で 0割り");
                return;
            }

            double ylen = (double)Setting.FMY / dnumy; // ylenずつ置けば良い

            if (t2 == 0)
                num007 = 0;
            if (t == 0)
            {
                double x = 0.0;
                double y = 0.0;
                double angle = 0.0;
                int cnum = 0;

                for (int j = 0; j < 4; j++) // 画面左上端から、右→下→左→上の 4方向移動させる
                {
                    int to = (j % 2 != 0) ? dnumy : DNUMX;
                    for (int i = 0; i <= to; i++) // 水平方向、垂直方向では置く個数が違うので to回置く
                    {
                        for (int s = 0; s < 2; s++)
                        {
                            Bullet bullet = boss_shot.search_boss_shot();
                            if (bullet != null)
                            {
                                bullet.x = x; // 座標
                                bullet.y = y;
                                bullet.knd = 4; // 種類
                                bullet.flag = 1;
                                bullet.cnt = 0;
                                switch (num007)
                                {
                                    case 0: // 直線
                                        bullet.col = 0; // 弾の色
                                        bullet.angle = angle + Math.PI / 2; // 角度
                                        bullet.spd = 1.1 + 0.5 * s; // スピード
                                        bullet.state = 0; // ステータス
                                        break;
                                    case 1: // 斜め交差
                                        bullet.col = 3; // 弾の色
                                        bullet.angle = angle + Math.PI / 2 - Math.PI / 14 + Math.PI / 7 * s; // 角度
                                        bullet.spd = 1.3;
                                        bullet.state = 1; // ステータス
                                        break;
                                }
                                MusicManager musicManager = MusicManager.GetInstance();
                                musicManager.se_flag[0] = 1;

                                cnum++;
                            }
                        }
                        x += Math.Cos(angle) * xlen;
                        y += Math.Sin(angle) * xlen;
                    }
                    angle += Math.PI / 2;
                }
            }

            if (t == TM007 - 1)
            {
                num007++;
                num007 = num007 % 2;
            }
        }

        /// <summary>
        /// 弾幕を処理します。必要に応じて弾を追加します。
        /// </summary>
        /// <param name="boss_shot">ボスの弾幕</param>
        public static void do_boss_shot_bullet(BossShot boss_shot)
        {
            switch (GV.boss.knd)
            {
                case 0:
                    boss_shot_bulletH007(boss_shot);
                    break;
                case 1:
                    boss_shot_bulletH000(boss_shot);
                    break;
                case 2:
                    boss_shot_bulletH000(boss_shot);
                    break;
                case 3:
                    boss_shot_bulletH000(boss_shot);
                    break;
                case 4:
                    boss_shot_bulletH000(boss_shot);
                    break;
                case 5:
                    boss_shot_bulletH000(boss_shot);
                    break;
                case 6:
                    boss_shot_bulletH000(boss_shot);
                    break;
                default:
                    Debug.WriteLine("boss.kndの値が不正(" + GV.boss.knd + ")です");
                    break;
            }
        }
    }
}
