﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵の出現情報群を管理するクラスです。
    /// </summary>
    class EnemyOrderGroup
    {
        /// <summary>
        /// 敵の出現情報群。
        /// </summary>
        private EnemyOrder[] enemy_orders = new EnemyOrder[Setting.ENEMY_ORDER_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public EnemyOrderGroup()
        {
            for (int i = 0; i < this.enemy_orders.Length; i++)
            {
                this.enemy_orders[i] = new EnemyOrder();
            }
        }

        /// <summary>
        /// 敵の出現情報にアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>敵の出現情報</returns>
        public EnemyOrder this[int index]
        {
            get
            {
                return this.enemy_orders[index];
            }
        }

        /// <summary>
        /// 敵の出現情報の数を返します。
        /// </summary>
        /// <returns>敵の出現情報の数</returns>
        public int length()
        {
            return this.enemy_orders.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.enemy_orders.Length; i++)
                this.enemy_orders[i].initialize();
        }

        /// <summary>
        /// 敵の出現情報を CSVファイルから読み込んで格納します。
        /// </summary>
        public void load_story()
        {
            string fname = Setting.BASE_DIR + "csv/45章/storyH0.csv";
            int n = 0; // 格納する配列番号
            int lineNumber = 0; // ファイルの行数

            DXFileReader reader = new DXFileReader();
            if(! reader.open(fname, 0))
            {
                Debug.WriteLine("ファイルオープンに失敗しました(" + fname + ")");
                return;
            }

            for (int i = 0; i < 2; i++) // 最初の 2行を読み飛ばす
            {
                reader.readLine();
                lineNumber++;
            }

            try {
                while (true)
                {
                    string line = reader.readLine(); // 1行読み込む
                    lineNumber++;

                    if (line == null) // ファイルの末端に到達していれば
                        break; // ループから抜ける

                    if (line[0] == '/') // スラッシュがあれば
                        continue; // 読み飛ばす

                    string[] items = line.Split(new Char[] { ',' }); // 文字列を分解する

                    // 情報を格納する
                    EnemyOrder enemy_order = GV.enemy_orders[n];
                    enemy_order.cnt = int.Parse(items[0]);
                    enemy_order.pattern = int.Parse(items[1]);
                    enemy_order.knd = int.Parse(items[2]);
                    enemy_order.x = double.Parse(items[3]);
                    enemy_order.y = double.Parse(items[4]);
                    enemy_order.sp = double.Parse(items[5]);
                    enemy_order.bltime = int.Parse(items[6]);
                    enemy_order.blknd = int.Parse(items[7]);
                    enemy_order.col = int.Parse(items[8]);
                    enemy_order.hp = int.Parse(items[9]);
                    enemy_order.blknd2 = int.Parse(items[10]);
                    enemy_order.wait = int.Parse(items[11]);
                    enemy_order.item_n[0] = int.Parse(items[12]);
                    enemy_order.item_n[1] = int.Parse(items[13]);
                    enemy_order.item_n[2] = int.Parse(items[14]);
                    enemy_order.item_n[3] = int.Parse(items[15]);
                    enemy_order.item_n[4] = int.Parse(items[16]);
                    enemy_order.item_n[5] = int.Parse(items[17]);

                    n++;
                }
            }
            catch (FormatException)
            {
                Debug.WriteLine(lineNumber + "行目に不適切なデータが存在します");
            }

            // ファイルのクローズ
            if (!reader.close())
            {
                Debug.WriteLine("ファイルのクローズに失敗しました(" + fname + ")");
            }
        }
    }
}
