﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 音楽を管理するためのクラスです。
    /// </summary>
    class MusicManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static MusicManager manager = new MusicManager();
        
        /// <summary>
        /// サウンドSE。
        /// </summary>
        private int[] sound_se = new int[Setting.SE_MAX];

        /// <summary>
        /// サウンドSEフラグ。
        /// </summary>
        public int[] se_flag = new int[Setting.SE_MAX];

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private MusicManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static MusicManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void ini()
        {
            for (int i = 0; i < this.se_flag.Length; i++)
                this.se_flag[i] = 0;
        }

        /// <summary>
        /// 音楽を鳴らします。
        /// </summary>
        public void play()
        {
            for (int i = 0; i < this.sound_se.Length; i++)
                if (this.se_flag[i] == 1)
                {
                    if(DX.CheckSoundMem(this.sound_se[i]) != 0)
                    {
                        /* ショット音の関係で暫定的にコメントアウト
                        if(i == 8) // 8番はショットが敵に当たった音の場合には
                            continue; // 音楽を止めない
                         */
                        DX.StopSoundMem(this.sound_se[i]);
                    }
                    DX.PlaySoundMem(this.sound_se[i], DX.DX_PLAYTYPE_BACK);
                }
        }
        
        /// <summary>
        /// 音楽データを読み込みます。
        /// </summary>
        public void load()
        {
            DX.SetCreateSoundDataType(DX.DX_SOUNDDATATYPE_MEMPRESS);

            string baseDir = Setting.BASE_DIR;
            this.sound_se[0] = DX.LoadSoundMem(baseDir + "se/enemy_shot.wav");
            this.sound_se[1] = DX.LoadSoundMem(baseDir + "se/enemy_death.wav");
            this.sound_se[2] = DX.LoadSoundMem(baseDir + "se/cshot.wav");
            this.sound_se[3] = DX.LoadSoundMem(baseDir + "se/char_death.wav"); // 自キャラクタの死に音
            this.sound_se[8] = DX.LoadSoundMem(baseDir + "se/hit.wav");
            this.sound_se[9] = DX.LoadSoundMem(baseDir + "se/boss_death.wav");
            this.sound_se[11] = DX.LoadSoundMem(baseDir + "se/boss_change.wav");
            this.sound_se[14] = DX.LoadSoundMem(baseDir + "se/bom1.wav");
            this.sound_se[15] = DX.LoadSoundMem(baseDir + "se/bom2.wav");
            this.sound_se[33] = DX.LoadSoundMem(baseDir + "se/lazer.wav");
            this.sound_se[34] = DX.LoadSoundMem(baseDir + "se/item_get.wav");

            // this.sound_se[99] = DX.LoadSoundMem(baseDir + "music/youmu.wav");

            // 各素材の再生ボリュームを設定
            DX.ChangeVolumeSoundMem(32, this.sound_se[0]);
            DX.ChangeVolumeSoundMem(32, this.sound_se[1]);
            DX.ChangeVolumeSoundMem(32, this.sound_se[2]);
            DX.ChangeVolumeSoundMem(32, this.sound_se[3]);
            DX.ChangeVolumeSoundMem(32, this.sound_se[8]);
            DX.ChangeVolumeSoundMem(64, this.sound_se[9]);
            DX.ChangeVolumeSoundMem(64, this.sound_se[11]);
            DX.ChangeVolumeSoundMem(64, this.sound_se[14]);
            DX.ChangeVolumeSoundMem(64, this.sound_se[15]);
            DX.ChangeVolumeSoundMem(64, this.sound_se[33]);
            DX.ChangeVolumeSoundMem(32, this.sound_se[34]);
        }

        /// <summary>
        /// 音楽を再生します。
        /// </summary>
        /// <param name="num">番号</param>
        public void play(int num)
        {
            DX.PlaySoundMem(this.sound_se[num], DX.DX_PLAYTYPE_BACK);
        }

        /// <summary>
        /// BGMを読み込みます。
        /// </summary>
        /// <param name="num">番号</param>
        public void load_bgm(int num){
            string baseDir = Setting.BASE_DIR;
            string str = baseDir + "music/" + GV.stage + "_" + num + ".ogg";
            GV.area.bgm.handle = DX.LoadSoundMem(str);
        }

        /// <summary>
        /// BGMを削除します。
        /// </summary>
        public void delete_bgm()
        {
            DX.DeleteSoundMem(GV.area.bgm.handle);
        }

        /// <summary>
        /// BGMのメイン処理です。
        /// </summary>
        public void bgm_main()
        {
            if (GV.area.bgm.flag == 0) // 最初に読み込む
            {
                this.load_bgm(0);
                DX.SetLoopPosSoundMem(GV.area.bgm.loop_pos[0], GV.area.bgm.handle); // ループ位置セット
                GV.area.bgm.flag = 1;
            }
            if (GV.area.bgm.flag == 1) // 種類に応じて再生をはじめる
            {
                switch (GV.area.bgm.knd[0])
                {
                    case 0: // 再生種類 0なら
                        if (GV.stage_count == 60) // はじまって 60カウント後に再生
                        {
                            DX.PlaySoundMem(GV.area.bgm.handle, DX.DX_PLAYTYPE_LOOP);
                            GV.area.bgm.flag = 2;
                        }
                        break;
                    default:
                        break;
                }
            }
            if (GV.area.bgm.flag == 2) // ボス出現時に読み込む
            {
                if (GV.stage_count == GV.boss.appear_count[1]) // ラスボスが現れたら読み込む
                {
                    DX.StopSoundMem(GV.area.bgm.handle);
                    this.delete_bgm();
                    this.load_bgm(1);
                    DX.SetLoopPosSoundMem(GV.area.bgm.loop_pos[1], GV.area.bgm.handle); // ループ位置セット
                    GV.area.bgm.flag = 3;
                }
            }
            if (GV.area.bgm.flag == 3)
            {
                switch (GV.area.bgm.knd[0])
                {
                    case 0: // 再生種類 0なら
                        if (GV.stage_count == GV.boss.appear_count[1]) // ラスボスが現れたら再生
                        {
                            DX.PlaySoundMem(GV.area.bgm.handle, DX.DX_PLAYTYPE_LOOP);
                            GV.area.bgm.flag = 4;
                        }
                        break;
                    default:
                        break;
                }
            }
        }
    }
}
