﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// メインルーチンクラスです。
    /// </summary>
    class Program
    {
        /// <summary>
        /// ループ処理。
        /// </summary>
        /// <returns>エラーなら -1、正常なら 0</returns>
        private static int ProcessLoop()
        {
            if (DX.ProcessMessage() != 0)
                return -1; // プロセス処理がエラーなら -1を返す
            if (DX.ClearDrawScreen() != 0)
                return -1; // 画面クリア処理がエラーなら -1を返す

            KeyManager keyManager = KeyManager.GetInstance();
            keyManager.GetHitKeyStateAll_2(); //現在のキー入力処理を行う
            keyManager.GetHitPadStateAll(); //現在のパッド入力処理を行う

            return 0;
        }

        /// <summary>
        /// エフェクトメイン。
        /// </summary>
        private static void effect_main()
        {
            GV.children.calc_child(); // 子どもの計算
            GV.dn.dn_calc(); // ドン揺れの計算
            GV.del_effects.calc_del_effect(); // 消滅エフェクトの計算
            GV.effects.calc_effect(); // エフェクトの計算
            GV.bom.bom_calc(); // ボムの計算
            GV.ch.enter_crybom_effect(); // 喰らいボムのエフェクト登録
        }

        /// <summary>
        /// 当たり判定メイン。
        /// </summary>
        private static void out_main()
        {
            GV.bom.cbom_and_enemy(); // 自キャラクタのボムが敵に当たる
            GV.cshots.cshot_and_enemy(); // 自キャラクタの弾が敵に当たる
            GV.lazers.lazer_and_ch(); // 敵のレーザーが自キャラクタに当たる
            GV.shots.enemyshot_and_ch(); // 敵弾が自キャラクタに当たる
            GV.boss_shot.bossshot_and_ch(); // ボス弾が自キャラクタに当たる(enemy_shot_and_chから分離独立した)
            GV.items.ch_and_item(); // アイテム接触処理
        }

        /// <summary>
        /// ゲームタイトル表示メイン。
        /// </summary>
        private static void calc_main()
        {
            GV.stage_title.calc_stage_title(); // ステージタイトル表示の計算
            GV.items.calc_item(); // アイテム群の計算
            GV.flash.calc_flash(); // フラッシュの計算
        }

        /// <summary>
        /// メインメソッド
        /// </summary>
        /// <param name="args">引数</param>
        public static void Main(string[] args)
        {
            Debug.Listeners.Add(new TextWriterTraceListener(Console.Out));
            Debug.WriteLine("起動しました。");

            CheckTime.initialize(); // 時間測定の初期化

            DX.ChangeWindowMode(1); // ウインドウモード

            // 初期化と裏画面化
            if (DX.DxLib_Init() == -1 || DX.SetDrawScreen(DX.DX_SCREEN_BACK) != 0)
                return;

            MusicManager musicManager = MusicManager.GetInstance();

            while (ProcessLoop() == 0)
            {
                musicManager.ini();
                switch (GV.func_state)
                {
                    case 0:
                        // データロード
                        ImageManager imageManager = ImageManager.GetInstance();
                        imageManager.load();
                        musicManager.load();
                        GV.first_ini(); // 初回の初期化
                        GV.func_state = 99;
                        break;
                    case 99:
                        GV.ini(); // ゲームの初期化
                        GV.enemy_orders.load_story(); // 敵の出現情報の読み込み
                        GV.func_state = 100;
                        break;
                    case 100:
                        CheckTime.enter_func_tm("最初");
                        GV.ch.calc_ch(); // キャラクタ計算
                        CheckTime.enter_func_tm("キャラ計算");
                        GV.ch.ch_move(); // キャラクタの移動制御
                        CheckTime.enter_func_tm("キャラ移動");
                        GV.cshots.cshot_main(); // 自キャラクタのショットメイン
                        CheckTime.enter_func_tm("自機ショットメイン");
                        GV.enemys.enemy_main(); // 敵処理メイン
                        CheckTime.enter_func_tm("敵処理メイン");
                        GV.boss.boss_shot_main(); // ボスのショットメイン
                        CheckTime.enter_func_tm("ボスショットメイン");
                        GV.shots.shot_main(); // 弾幕メイン
                        GV.lazers.lazer_main(); // レーザーメイン
                        CheckTime.enter_func_tm("ショットメイン");
                        out_main(); // 当たり計算メイン
                        CheckTime.enter_func_tm("当たり判定");
                        effect_main(); // エフェクトメイン
                        CheckTime.enter_func_tm("エフェクトメイン");
                        calc_main(); // 表示計算
                        CheckTime.enter_func_tm("計算メイン");

                        GraphManager graphManager = GraphManager.GetInstance();
                        graphManager.graph_main(); // 描画メイン
                        CheckTime.enter_func_tm("描画メイン");

                        musicManager.bgm_main(); // BGMメイン
                        CheckTime.enter_func_tm("BGMメイン");

                        if(GV.boss.flag == 0)
                            GV.increment_stage_count();
                        break;
                    default:
                        Debug.WriteLine("不明な func_state");
                        break;
                }
                musicManager.play();
                CheckTime.enter_func_tm("音楽再生");
                Fps.fps_wait();
                CheckTime.enter_func_tm("待機した時間", 1);

                KeyManager keyManager = KeyManager.GetInstance();
                if (keyManager.CheckStateKey(DX.KEY_INPUT_ESCAPE) == 1)
                    break; //エスケープが入力されたらブレイク

                DX.ScreenFlip(); // 裏画面反映
                GV.incrementCount();
            }

            DX.DxLib_End(); // DXライブラリ終了処理
        }
    }
}
