﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 消滅エフェクト群を管理するクラスです。
    /// </summary>
    class DelEffectGroup
    {
        /// <summary>
        /// 消滅エフェクト群。
        /// </summary>
        private DelEffect[] del_effects = new DelEffect[Setting.DEL_EFFECT_MAX];

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public DelEffectGroup()
        {
            for (int i = 0; i < this.del_effects.Length; i++)
            {
                this.del_effects[i] = new DelEffect();
            }
        }

        /// <summary>
        /// 消滅エフェクトにアクセスするインデクサ。
        /// </summary>
        /// <param name="index">番号</param>
        /// <returns>消滅エフェクト</returns>
        public DelEffect this[int index]
        {
            get
            {
                return this.del_effects[index];
            }
        }

        /// <summary>
        /// 消滅エフェクトの数を返します。
        /// </summary>
        /// <returns>消滅エフェクトの数</returns>
        public int length()
        {
            return this.del_effects.Length;
        }

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            for (int i = 0; i < this.del_effects.Length; i++)
                this.del_effects[i].initialize();
        }
        
        /// <summary>
        /// 消滅用エフェクトの登録空き番号を探します。
        /// </summary>
        /// <returns>消滅用エフェクト。無い場合は null</returns>
        private DelEffect search_del_effect()
        {
            for (int i = 0; i < this.del_effects.Length; i++)
            {
                DelEffect del_effect = this.del_effects[i];
                if (del_effect.flag == 0)
                    return del_effect;
            }

            return null;
        }
        
        /// <summary>
        /// 消滅エフェクトを登録します。
        /// </summary>
        /// <param name="enemy">敵</param>
        public void enter_del_effect(Enemy enemy)
        {
            DelEffect del_effect = this.search_del_effect();
            if (del_effect != null)
            {
                del_effect.flag = 1; // フラグを立てる
                del_effect.cnt = 0;
                del_effect.col = enemy.back_col; // 敵の背景色を消滅色に設定する
                del_effect.x = enemy.x; // 敵の座標を消滅位置にセット
                del_effect.y = enemy.y;
            }
        }

        /// <summary>
        /// 消滅エフェクトを計算します。エフェクトの登録も行います。
        /// </summary>
        public void calc_del_effect()
        {
            for (int i = 0; i < this.del_effects.Length; i++)
            {
                DelEffect del_effect = this.del_effects[i];
                if (del_effect.flag > 0)
                    del_effect.calc_del_effect();
            }
        }
    }
}
